"""

Python Desktop Server - Blogmark Editor

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: BlogmarkTool.py,v 1.32 2004/10/25 18:49:31 gb Exp $

import PyDS.Tool
import metakit
import time
import string
import sys
from WikiTool import _munge

linkListTemplate = """<table class="smallfont" border="0">
#for $item in $linklist
<tr><td><a href="$item.link" title="$item.title">$item.text</a></td></tr>
#end for
</table>"""

class BlogmarkTool(PyDS.Tool.StandardTool):

	def _recachepostings(self):
		try:
			self._acquire()
			self.postsByPubdate = self.posts.sort(self.posts.pubdate)
			self.postsByPubtime = self.posts.sort(self.posts.pubtime)
		finally: self._release()
	
	def _remove(self):
		PyDS.Tool.StandardTool._remove(self)
		del self.postsByPubdate
		del self.postsByPubtime

	def _createdb(klass, db):
		db.getas("posts[id:S,title:S,link:S,text:S,onhome:I,pubdate:S,pubtime:F,categories[name:S]]")
		db.getas("prefs[maxid:I,title:S,desc:S]")
		db.getas("status[setCategoriesActive:I]")
		db.getas("categories[name:S,title:S,desc:S,active:I]")
	
	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.posts = self.db.view('posts').ordered(1)
			self.prefs = self.db.view('prefs')
			self.status = self.db.view('status')
			self.categories = self.db.view('categories').ordered(1)
			dirty = 0
			if len(self.prefs) == 0:
				self.prefs.append(maxid=1)
				dirty = 1
			if len(self.status) == 0: 
				self.status.append(setCategoriesActive=0)
				dirty = 1
			if dirty:
				self._commit()
			self._recachepostings()
		finally: self._release()
	
	def _backup(self):

		def _splitpubtime(liste):
			for (cn, cv) in liste:
				if cn == 'pubtime':
					(y,m,t) = time.localtime(cv)[:3]
					return [
						('year', y),
						('month', m)
					]
			return []

		l = self._backupTable('prefs')
		l.extend(self._backupTable('categories'))
		l.extend(self._backupTable('posts', sortcols=['pubtime'], groupfn=_splitpubtime))
		return l
			
	def _initopts(self):
		self.rss = 1
		try:
			self._acquire()
			if not self.status[0].setCategoriesActive: 
				for cat in self.categories: 
					cat.active = 1
				self.status[0].setCategoriesActive = 1
				self._commit()
		finally: self._release()
		nuggets = self.getToolByNamespace('nuggets')
		nuggets.addStandardNugget('linkListBox',
			_('Template for the list of links from blogmarks'),
			linkListTemplate)
	
	def _initactive(self):
		themes = self.getToolByNamespace('themes')
		navigation = self.getToolByNamespace('navigation')
		weblog = self.getToolByNamespace('weblog')
		themes.hookTemplate(
			'BlogmarkTemplate',
			_('blogmark template for the desktop website'),
			'templates/BlogmarkTemplate.tmpl'
		)
		themes.hookTemplate(
			'BlogmarkRendering',
			_('blogmark template for the cloud'),
			'templates/BlogmarkRendering.tmpl'
		)
		navigation._addHook('select', 'blogmarkcat',
			_('Link to a blogmark category'),
			self._categoryNavHook,
			self.hasCategories)
		navigation._addHook('select', 'blogmarks',
			_('Link to the blogmarks main index'),
			self._indexNavHook,
			None)
		weblog._addHook('rssitems', 'blogmarks',
		 	'',
			self._rssItemsForWeblog,
			self._rssItemsForWeblogActive)
		
	def _initthread(self, server):
		while server.isRunning():
			time.sleep(10)
			if self.queue.length():
				while self.queue.length():
					(jobf, key, data) = self.queue.shift()
					jobf(key, data)
				self.logVerbose(_('work done, going to sleep again'))

	def _resync(self, theMirror):
		for post in self.posts:
			if post.onhome or len(post.categories):
				theMirror.iHave(self, post.id)

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d categories defined</li>') % len(self.categories)
			html += _('<li>There are %d postings in the database</li>') % len(self.posts)
			html += _('<li>There are %d postings in the render queue</li>') % self.queue.length()
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
	def _rssItemsForWeblog(self, category):
		items = None
		(year, month, day) = (time.localtime(time.time()))[:3]
		if category == '':
			items = self.getRecentPosts(maxnumber=20, onhome=1, category='')
		else:
			items = self.getRecentPosts(maxnumber=20, onhome=0, category=category)
		res = []
		for item in items:
			el = self._translateItemToRSS(item, category)
			res.append(el)
		return res
	
	def _rssItemsForWeblogActive(self, category):
		if category == '':
			return 1
		else:
			return self.hasCategory(category)
	
	def _categoryNavHook(self, qual, value, preset=None):
		if qual == 'html':
			html = '<select name="%s">' % value
			for cat in self.getCategories():
				selected = ''
				if cat['name'] == preset:
					selected = ' selected'
				html += '<option value="%s"%s>%s' % (
					cat['name'],
					selected,
					cat['title']
				)
			html += '</select>'
			return html
		elif qual == 'url':
			return self.getStaticUrl(category=value)
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _indexNavHook(self, qual, value, preset=None):
		if qual == 'html':
			return ''
		elif qual == 'url':
			return self.getStaticUrl()
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _search(self, query):
		try:
			self._acquire()
			query = query.lower()
			macros = self.getToolByNamespace('macros')
			liste = []
			for post in self.postsByPubtime:
				if post.title.lower().find(query) >= 0 or post.text.lower().find(query) >= 0:
					guid = ''
					if post.onhome:
						guid = self.getCloudUrlForItem(post.pubtime, post.id)
					elif len(post.categories):
						category = post.categories[0].name
						guid = self.getCloudUrlForItem(post.pubtime, post.id, category=category)
					desc = macros.stripHTML(post.text)
					if len(desc) > 200:
						desc = desc[:200] + ' ...'
					liste.append({
						'title': post.title,
						'link': self._getDummyRequest(now=post.pubtime).getUrl(),
						'guid': guid,
						'description': desc
					})
			liste.reverse()
			return liste
		finally: self._release()

	# This methods gives you an cloud URL for a day. You can pass in the
	# reference time (as float) and an optional category name.
	def getCloudUrlForDay(self, reftime, category=''):
		(year, month, day) = time.localtime(reftime)[:3]
		return self.getStaticUrl(year=year, month=month, day=day, category=category)
					
	# This methods gives you an cloud URL for a posting. You can pass in the
	# reference time (as float) and an optional category name. This item
	# is referenced as a fragment of a day page, so you pass in a time
	# value and the ID of the posting. You additionally can pass in
	# a category name, if you want to link to the category day page.
	def getCloudUrlForItem(self, reftime, id, category=''):
		url = self.getCloudUrlForDay(reftime, category)
		url += '#%s' % id
		return url
		
	# --------------------------------------------------------------------
	# NEW STRAY LINK RESOLUTION METHODS
	# --------------------------------------------------------------------

	# Find items potentially matching the link text, and return their unique IDs. 
	def getUniqueIdsMatchingLinkText(self, linkText): 
		"""Find items potentially matching `linkText`. 
		Return a list of string IDs uniquely identifying those items."""
		 
		if not linkText: 
			return # never match empty link text

		# precalculate more permissive versions of the link text
		m = _munge(linkText)
		lt = linkText.lower()

		# precalculate permissive versions of context-dependent variables
		conid = _flet.get('uniqueId', '').lower() # uniqueId
		concatid = _flet.get('categoryid', '').lower() # categoryid
		
		# define filters, each of which depends on local variables defined 
		# previously -- a lot easier than passing lots of arguments, no?
		orderedFilters = (
			# id match to link text
			lambda: lid == lt, 
			# title match
			lambda: m == _munge(post.title()),
			)

		# prepare filter results dictionary
		filterResults = {}
		for filter in orderedFilters: 
			filterResults[filter] = []

		# run each post through the filters
		# if CPU becomes a bigger worry than memory, we can cache all these
		try: 
			self._acquire()
			for post in self.posts: 
				# precalculate more permissive version of post attributes
				nid = post.id
				lid = nid.lower()
				# execute the filters
				for filter in orderedFilters: 
					try: 
						if filter(): 
							filterResults[filter].append(nid)
					except: 
						pass # not worried about filter failure
		finally:
			self._release()

		# gather results into a single list, preserving order
		allResults = []
		inAllResults = {}
		for filter in orderedFilters: 
			for result in filterResults[filter]: 
				if inAllResults.has_key(result): 
					continue
				else: 
					inAllResults[result] = 1
					allResults.append(result)
		return allResults

	# Return the upload target for a given item in the current context. 
	def getCloudLinkDetailsForUniqueId(self, uniqueId): 
		"""Return the upload target for `uniqueId` in the current context.
		Raise KeyError if `uniqueId` cannot be found."""
		post = self.getPost(uniqueId)
		if post:
			categories = post.get('categories', [])
			if len(categories): 
				cat = categories[0]
			else: 
				cat = ''
			return ('blogmark', 
			        self.getCloudUrlForItem(post['pubtime'],
			                                id=uniqueId, 
			                                category=cat), 
			        post.get('title', ''))
		else: 
			raise KeyError, uniqueId

	# Return a URL for an edit page for a given item.
	def getDesktopDisplayLinkDetailsForUniqueId(self, uniqueId): 
		"""Return the edit URL for `uniqueId`."""
		post = self.getPost(uniqueId)
		if post:
			return ('blogmark', self.getUrl('edit_html', pid=uniqueId), post.get('title',''))
		else: 
			raise KeyError, uniqueId

    # Return a link to a creation page for an item, based on link text. 
	def getDesktopCreateLinkDetailsForProposedLinkText(self, linkText): 
		"""Return the edit URL for a non-existent item, based on `linkText`."""
		return ('undefinedpost', 
		        self.getUrl('edit_html', id=_munge(linkText)),
			_("Create this item in BlogmarkTool"))

	# Render an item by its unique ID
	def renderItemByUniqueId(self, uniqueId):
		"""Render all pages associated with the post identified by `uniqueId`."""
		post  = getPost(uniqueId)
		pubtime = post['pubtime']
		if post['onhome']: 
			self.renderTimeframes(pubtime, '')
		for cat in post['categories']: 
			self.renderTimeframes(pubtime, cat)
    
	# --------------------------------------------------------------------

	# this method returns the title of a page for rendering. This does a
	# lookup of data in the category or in the preferences and adds
	# date specifiers. The context passed in is the context that you
	# get in your template.
	def getPageTitle(self, context):
		title = self.getPref('title')
		if not(title):
			title = PyDS.Tool.StandardTool.getPageTitle(self, context)
		if context.get('category',''):
			cat = self.getCategory(context['category'])
			if cat:
				title = cat['title']
		if context.get('year',0) and context.get('month',0) and context.get('day',0):
			title += ' %d.%d.%d' % (context['day'], context['month'], context['year'])
		elif context.get('year',0) and context.get('month',0):
			title += ' %d/%d' % (context['year'], context['month'])
		return title
		
	# this method returns the description of a page for rendering. It's
	# a sister to the getPageTitle method.
	def getPageDescription(self, context):
		desc = self.getPref('desc')
		if not(desc):
			desc = PyDS.Tool.StandardTool.getPageDescription(self, context)
		if context.get('category',''):
			cat = self.getCategory(context['category'])
			if cat:
				desc = cat['desc']
		return desc
		
	def index_html(self, req):
		ptext = req.getFirstValue('ptext')
		ptextshort = ptext
		if len(ptext) > 200:
			ptextshort = ptext[:200] + ' ...'
		if not(req.getFirstValue('withtext')):
			ptext = ''
		html = req.renderPage('BlogmarkTemplate',
			mode='index',
			pid='',
			ptitle=req.getFirstValue('ptitle'),
			plink=req.getFirstValue('plink'),
			ptextshort=ptextshort,
			ptext=ptext,
			ponhome=req.getFirstValue('ponhome'),
			listblogmarks=1
		)
		return req.renderPage('BaseTemplate', body=html)

	def new_html(self, req):
		ptext = req.getFirstValue('ptext')
		ptextshort = ptext
		if len(ptext) > 200:
			ptextshort = ptext[:200] + ' ...'
		if not(req.getFirstValue('withtext')):
			ptext = ''
		html = req.renderPage('BlogmarkTemplate',
			mode='index',
			pid='',
			ptitle=req.getFirstValue('ptitle'),
			plink=req.getFirstValue('plink'),
			ptextshort=ptextshort,
			ptext=ptext,
			ponhome=req.getFirstValue('ponhome'),
			listblogmarks=0
		)
		return req.renderPage('BasePopup', body=html, onload='document.forms.blogmarkForm.ptext.focus()')
	
	def done_html(self, req):
		return req.renderPage('BasePopup', body=_('<p>The blogmark was added successfully.'), onload='')

	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('pid')
		(res, found) = self.posts.locate({'id':id})
		if found:
			row = self.posts[res]
			textshort = macros.stripHTML(row.text)
			if len(textshort) > 200:
				textshort = textshort[:200] + ' ...'
			html = req.renderPage('BlogmarkTemplate',
				mode='edit',
				pid=row.id,
				ptitle=row.title,
				plink=row.link,
				ptextshort=textshort,
				ptext=row.text,
				ptime=row.pubtime,
				pcategories=map(lambda c: c.name, row.categories),
				ponhome=row.onhome,
				listblogmarks=1
			)
			return req.renderPage('BaseTemplate', body=html)
		else:
			return self.index_html(req)

	def delete_html(self, req):
		id = req.getFirstValue('pid')
		try:
			self._acquire()
			(res, found) = self.posts.locate({'id':id})
			if found:
				row = self.posts[res]
				html = req.renderPage('BlogmarkTemplate',
					mode='delete',
					pid=row.id,
					ptitle=row.title,
					plink=row.link,
					ptext=row.text,
					listblogmarks=1
				)
				return req.renderPage('BaseTemplate', body=html)
			else:
				return self.index_html(req)
		finally:
			self._release()

	def refresh_redir(self, req):
		pid = req.getFirstValue('pid')
		try:
			self._acquire()
			(res, found) = self.posts.locate({'id':pid})
			if found:
				post = self.posts[res]
				if post.onhome:
					self.renderTimeframes(post.pubtime, category='')
				for cat in post.categories:
					self.renderTimeframes(post.pubtime, category=cat.name)
				return req.getUrl()
			else:
				return req.getUrl(errmsg=_('posting %s not found') % pid)
		finally:
			self._release()

	def post_redir(self, req):
		title = req.getFirstValue('ptitle')
		text = req.getFirstValue('ptext')
		ponhome = 0
		if req.getFirstValue('ponhome'):
			ponhome = 1
		categories = []
		categories.extend(req.getAllValues('pcategories'))
		pextracategories = req.getFirstValue('pextracategories')
		if pextracategories:
			categories.extend(self._addExtraCategories(pextracategories))
		id = self.postNewEntry(
			title=title,
			link=req.getFirstValue('plink'),
			text=text,
			onhome=ponhome,
			categories=categories
		)
		if req.dateGiven:
			ot = list(time.gmtime(time.time()))
			ot[0] = req.year
			ot[1] = req.month
			if req.dayGiven:
				ot[2] = req.day
			otime = time.mktime(ot)
			self.editOldEntry(id, pubtime=otime)
		for hook in self._getHooks('postnew'):
			if hook.isActive(req):
				hook(self, req, id)
		if int(req.getFirstValue('listblogmarks') or '0'):
			return req.getUrl()
		else:
			return req.getUrl('done_html')

	def edit_redir(self, req):
		ponhome = 0
		if req.getFirstValue('ponhome'):
			ponhome = 1
		pid = req.getFirstValue('pid')
		categories=[]
		categories.extend(req.getAllValues('pcategories'))
		pextracategories = req.getFirstValue('pextracategories')
		if pextracategories:
			categories.extend(self._addExtraCategories(pextracategories))
		self.editOldEntry(
			id=pid,
			title=req.getFirstValue('ptitle'),
			link=req.getFirstValue('plink'),
			text=req.getFirstValue('ptext'),
			categories=categories,
			onhome=ponhome
		)
		for hook in self._getHooks('editold'):
			if hook.isActive(req):
				hook(self, req, pid)
		return req.getUrl()

	def delete_redir(self, req):
		pid = req.getFirstValue('pid')
		for hook in self._getHooks('deleteold'):
			if hook.isActive(req):
				hook(self, req, pid)
		self.deleteOldEntry(id=pid)
		return req.getUrl()
		
	# This method delivers the description of your preferences panels
	# to hook up into the preferences list.
	def getPreferencePanels(self):
		return [
			['categories_panel', _('Blogmark Categories'),
				_('You can configure your blogmark categories here.')]
		]
			
	def categories_panel(self, req):
		macros = self.getToolByNamespace('macros')
		html = '<p>%s' % _("Enter the title and description of your blogmarks main category here.")
		html += '<form action="%s" method="POST">' % req.getUrl('prefs_redir', tool='blogmark')
		html += '<table border=0 width="100%">'
		html += '<tr><td>%s</td><td>' % _('blogmarks title')
		html += '<input name="blogmarkstitle" size=30 value="%s">' % self.getPref('title')
		html += '</td></tr><tr><td>%s</td><td>' % _('blogmarks description')
		html += '<textarea name="blogmarksdesc" cols=30 rows=5>%s</textarea>' % macros.quoteHTML(self.getPref('desc'))
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="CategorySubmit" value="%s">' % _('Save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p>%s' % _("Here you can add or delete categories for your blogmarks.")
		html += '<form action="%s" method="POST">' % req.getUrl('addcategory_redir', tool='blogmark')
		html += '<table border=0 width="100%">'
		catname = req.getFirstValue('catname')
		cattitle = ''
		catdesc = ''
		catactive = 1
		for cat in self.getCategories():
			if cat['name'] == catname:
				cattitle = cat['title']
				catdesc = cat['desc']
				catactive = cat['active']
			html += '<tr><td colspan=2 class="whitebox">'
			html += '<table border=0 width="100%"><tr><td width="80%">'
			html += '<b>%s(%s):</b> %s [%s]' % (
				cat['title'],
				cat['name'],
				cat['desc'],
				[_('inactive'), _('active')][cat['active']]
			)
			html += '</td><td width="20%" align="right" valign="top">'
			html += '%s<br>%s</td></tr>' % (
				'<a href="%s">%s</a>' % (
					req.getUrl(
						'panel_html',
						tool='preferences',
						panel='categories_panel',
						fortool='blogmark',
						catname=cat['name']
					), _('edit')
				),
				'<a href="%s">%s</a>' % (
					req.getUrl(
						'delcategory_redir',
						tool='blogmark',
						catname=cat['name']
					), _('delete')
				)
			)
			html += '</table></td></tr>'
		html += '<tr><td>%s</td><td>' % _('category name')
		html += '<input name="catname" size=10 value="%s">' % catname
		html += '</td></tr><tr><td>%s</td><td>' % _('category title')
		html += '<input name="cattitle" size=30 value="%s">' % cattitle
		html += '</td></tr><tr><td>%s</td><td>' % _('active')
		html += '<input name="catactive" type="checkbox" value="1"%s>' % [
			'', ' checked'][not not catactive]
		html += '</td></tr><tr><td>%s</td><td>' % _('description')
		html += '<textarea name="catdesc" cols=40 rows=5>%s</textarea>' % macros.quoteHTML(catdesc)
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="CategorySubmit" value="%s">' % _('add')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		return html
	
	def prefs_redir(self, req):
		title = req.getFirstValue('blogmarkstitle')
		desc = req.getFirstValue('blogmarksdesc')
		try:
			self._acquire()
			self.prefs[0].title = title
			self.prefs[0].desc = desc
			self._commit()
		finally: self._release()
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='categories_panel',
			fortool='blogmark'
		)
		
	def addcategory_redir(self, req):
		name = req.getFirstValue('catname')
		title = req.getFirstValue('cattitle')
		desc = req.getFirstValue('catdesc')
		active = not not req.getFirstValue('catactive')
		self.addCategory(name, title, desc, active)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='categories_panel',
			fortool='blogmark'
		)
		
	def delcategory_redir(self, req):
		name = req.getFirstValue('catname')
		self.delCategory(name)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='categories_panel',
			fortool='blogmark'
		)

	# get the most recent postings for some date selection. Only when
	# you give an explicit day, you get all postings, otherwise you get
	# maximum maxnumber postings. You can filter by the category
	# or the status wether a posting shows on the homepage.
	def getRecentPosts(self, year=None, month=None, day=None, maxnumber=30, onhome=0, category=None):
		try:
			self._acquire()
			liste = []
			l = self.postsByPubtime
			if onhome:
				l = l.select({'onhome':1})
			if year and month and day:
				l = l.select({'pubdate':'%04d-%02d-%02d' % (year, month, day)})
			elif year and month:
				l = l.select(
					{'pubdate':'%04d-%02d-%02d' % (year, month, 1)},
					{'pubdate':'%04d-%02d-%02d' % (year, month, 31)}
				)
			idxl = range(0,len(l))
			idxl.reverse()
			count = 0
			for i in idxl:
				p = l[i]
				cats = []
				catfound = 0
				for c in p.categories:
					cats.append(c.name)
					if c.name == category:
						catfound = 1
				if category and not(catfound):
					continue
				post = {}
				post['id'] = p.id
				post['title'] = p.title
				post['link'] = p.link
				post['text'] = p.text
				post['categories'] = cats
				post['pubdate'] = p.pubdate
				post['pubtime'] = p.pubtime
				post['onhome'] = p.onhome
				liste.append(post)
				count += 1
				if count >= maxnumber and not(day):
					break
			return liste

		finally: self._release()

	# post a new entry to the blogmark. Pass in all components
	# of a posting. Categories can be passed in as a list
	# of category names.
	def postNewEntry(self, title, link, text, onhome, categories=[], pubtime=None):
		macros = self.getToolByNamespace('macros')
		calendar = self.getToolByNamespace('calendar')
		mirror = self.getToolByNamespace('mirror')
		try:
			self._acquire()
			id = self.prefs[0].maxid + 1
			self.prefs[0].maxid = id
			id = 'BM%d' % id
			if not(pubtime):
				pubtime = time.time()
	
			pubdate = macros.Today(now=pubtime)
			
			newPosting = {
				'id': id,
				'title': title,
				'link': link,
				'text': text,
				'categories': metakit.view(),
				'pubdate': pubdate,
				'pubtime': pubtime,
				'onhome': onhome
			}
			self.posts.append(newPosting)
			(res, found) = self.posts.locate({'id':id})
			if found:
				self.posts[res].categories = map(lambda c: {'name':c}, categories)
			self._commit()
			self._recachepostings()
			calendar.recache(self, reftime=pubtime)
			if onhome:
				calendar.recache(self, category='', reftime=pubtime)
				self.renderTimeframes(pubtime, category='')
			for cat in categories:
				calendar.recache(self, category=cat, reftime=pubtime)
				self.renderTimeframes(pubtime, category=cat)
			if onhome or len(categories): mirror.addItem(self, id)
			return id
		finally: self._release()
		
	# change some properties of an existing entry in the blogmark.
	# only properties that are not None are changed.
	def editOldEntry(self, id, title=None, link=None, text=None, categories=None, pubtime=None, onhome=None):
		try:
			self._acquire()
			calendar = self.getToolByNamespace('calendar')
			macros = self.getToolByNamespace('macros')
			mirror = self.getToolByNamespace('mirror')
			pubdate = macros.Today(now=pubtime)
			oldtime = 0
			(res, found) = self.posts.locate({'id':id})
			if found:
				row = self.posts[res]
				timeframe = row.pubtime
				if title != None:
					row.title = title
				if link != None:
					row.link = link
				if text != None:
					row.text = text
				oldcategories = None
				if categories != None:
					oldcategories = map(lambda c: c.name, row.categories)
					row.categories = map(lambda c: {'name':c}, categories)
				else:
					categories = map(lambda c: c.name, row.categories)
				if pubtime != None:
					pubtime = float(pubtime)
					row.pubdate = macros.Today(now=pubtime)
					row.pubtime = pubtime
					timeframe = pubtime
				oldhome = row.onhome
				oldvisible = oldhome or len(oldcategories)
				if onhome != None:
					row.onhome = onhome
				visible = row.onhome or len(categories)
				self._commit()
				self._recachepostings()
				calendar.recache(self, reftime=timeframe)
				if oldcategories:
					for cat in oldcategories:
						if not(cat in categories):
							categories.append(cat)
				if onhome != None:
					calendar.recache(self, category='', reftime=timeframe)
					self.renderTimeframes(timeframe, category='')
				for cat in categories:
					calendar.recache(self, category=cat, reftime=timeframe)
					self.renderTimeframes(timeframe, category=cat)
				if oldtime:
					calendar.recache(self, reftime=oldtime)
					if onhome:
						calendar.recache(self, category='', reftime=oldtime)
						self.renderTimeframes(oldtime, category='')
					for cat in categories:
						calendar.recache(self, category=cat, reftime=oldtime)
						self.renderTimeframes(oldtime, category=cat)
				if oldvisible:
					if visible: mirror.updateItem(self, id)
					else: mirror.deleteItem(self, id)
				else:
					if visible: mirror.addItem(self, id)
				return id
			else:
				raise ValueError(_("Invalid Posting id: %s") % id)
		finally: self._release()

	# delete an existing entry in the blogmark referenced by id.
	def deleteOldEntry(self, id):
		try:
			self._acquire()
			calendar = self.getToolByNamespace('calendar')
			mirror = self.getToolByNamespace('mirror')
			(res, found) = self.posts.locate({'id':id})
			if found:
				row = self.posts[res]
				timeframe = row.pubtime
				categories = map(lambda c: c.name, row.categories)
				visible = row.onhome or len(categories)
				self.posts.delete(res)
				self._commit()
				self._recachepostings()
				calendar.recache(self, reftime=timeframe)
				calendar.recache(self, category='', reftime=timeframe)
				self.renderTimeframes(timeframe, category='')
				for cat in categories:
					calendar.recache(self, category=cat, reftime=timeframe)
					self.renderTimeframes(timeframe, category=cat)
				if visible: mirror.deleteItem(self, id)
				return id
			else:
				raise ValueError(_("Invalid Posting id: %s") % id)
		finally: self._release()
		
	# get a posting by ID
	def getPost(self, id):
		try:
			self._acquire()
			(res,found) = self.posts.locate({'id':id})
			if found:
				post = self.posts[res]
				return {
					'id': post.id,
					'title': post.title,
					'link': post.link,
					'text': post.text,
					'pubdate': post.pubdate,
					'pubtime': post.pubtime,
					'onhome': post.onhome,
					'categories': map(lambda row: row.name, post.categories),
				}
			else:
				raise ValueError(_('Invalid Posting id: %s') % id)
		finally: self._release()
	
	def _addExtraCategories(self, str):
		liste = []
		for w in str.split(','):
			word = w.strip()
			if word and not self.getCategory(word):
				self.addCategory(
					word.lower(),
					'%s - %s' % (self.getPageTitle({}), word),
					_('Category "%s"') % word,
					active=1
				)
				liste.append(word.lower())
		return liste

	# add a new category or change it, if it already exists.
	def addCategory(self, name, title, desc, active=0):
		try:
			self._acquire()
			(res, found) = self.categories.locate({'name':name})
			if found:
				row = self.categories[res]
				row.title = title
				row.desc = desc
				row.active = active
			else:
				self.categories.append({
					'name':name, 
					'title':title, 
					'desc':desc, 
					'active':active})
			self._commit()
			return name
		finally: self._release()
	
	# delete an existing category.
	def delCategory(self, name):
		try:
			self._acquire()
			(idx, found) = self.categories.locate({'name':name})
			if found:
				self.categories.delete(idx)
				self._commit()
			return name
		finally: self._release()
	
	# check wether a given category exists
	def hasCategory(self, name):
		(idx, found) = self.categories.locate({'name':name})
		return found
		
	# get the category record given by name.
	def getCategory(self, name):
		try:
			self._acquire()
			(idx, found) = self.categories.locate({'name':name})
			if found:
				row = self.categories[idx]
				res = {
					'name':row.name,
					'title':row.title,
					'desc':row.desc,
					'active':row.active
				}
				return res
			return None
		finally: self._release()
	
	# get a list of categories
	def getCategories(self):
		try:
			self._acquire()
			liste = []
			for cat in self.categories:
				liste.append({
					'name':cat.name,
					'title':cat.title,
					'desc':cat.desc,
					'active':cat.active
				})
			return liste
		finally: self._release()
		
	# do we have categories defined? Returns the number of defined
	# categories.
	def hasCategories(self):
		try:
			self._acquire()
			return len(self.categories)
		finally: self._release()
		
	# get a list of days where you have postings. May be category restricted.
	# if you pass None as the category, you will get all postings. If you
	# pass in an empty string as the category, you get postings on the homepage.
	def getItemDays(self, category):
	
		def filterCategory(row):
			return category in map(lambda row: row.name, row.categories)
			
		def splitPubdate(pubdate):
			l = pubdate.split('-')
			return map(int, l)
			
		liste = self.posts
		if category:
			liste = filter(filterCategory, liste)
		elif category == '':
			liste = self.posts.select({'onhome':1})
		pubdates = {}
		for row in liste:
			pubdates[row.pubdate] = 1
		res = pubdates.keys()
		res.sort()
		return map(splitPubdate, res)
		
	# check wether the given date has any posting at all for the category.
	# Category None is all postings and an empty string for the category
	# refers to the homepage postings.
	def hasItemsForDay(self, year, month, day, category):
		pubdate = '%04d-%02d-%02d' % (year, month, day)
		if category:
			liste = self.postsByPubdate.select({'pubdate':pubdate})
			for el in liste:
				if category in map(lambda row: row.name, el.categories):
					return 1
			return 0
		elif category == '':
			(res, found) = self.postsByPubdate.locate({'pubdate':pubdate, 'onhome':1})
			return found > 0
		else:
			(res, found) = self.postsByPubdate.locate({'pubdate':pubdate})
			return found > 0

	def _rerender(self):
		try:
			self._acquire()
			cats = map(lambda row: row.name, self.categories)
			count = 0
			for post in self.posts:
				self.renderTimeframes(post.pubtime, '')
				count += 1
				for cat in post.categories:
					if cat.name in cats:
						self.renderTimeframes(post.pubtime, cat.name)
						count += 1
			return count
		finally: self._release()

	# This method background-renders all pages that are overlapping
	# a given reference time. This is the day page, the month page
	# and the main homepage. Additionally the previous month
	# is rendered, because the "next month" link might have changed.
	def renderTimeframes(self, basetime, category=''):
		weblog = self.getToolByNamespace('weblog')
		(year, month, day) = (time.localtime(basetime))[:3]
		self.renderTimeframe(year, month, day, category)
		self.renderTimeframe(year, month, 0, category)
		if category == '':
			weblog.renderTimeframe(year, month, day)
			weblog.renderTimeframe(year, month, 0)
		elif weblog.getCategory(category):
			weblog.renderTimeframe(year, month, day, category=category)
			weblog.renderTimeframe(year, month, 0, category=category)
		yp = year
		mp = month
		mp -= 1
		if mp == 0:
			mp = 12
			yp -= 1
		self.renderTimeframe(yp, mp, 0, category)
		self.renderTimeframe(0, 0, 0, category)
		if category == '':
			weblog.renderTimeframe(yp, mp, 0)
			weblog.renderTimeframe(0, 0, 0)
		elif weblog.getCategory(category):
			weblog.renderTimeframe(yp, mp, 0, category=category)
			weblog.renderTimeframe(0, 0, 0, category=category)
		return self.queue.length()

	# this method renders a single page in the background.
	def renderTimeframe(self, year, month, day, category=''):
		self.queue.append(self.renderTimeframeOnline, (year, month, day, category), None)
		return self.queue.length()
	
	# this method renders a single page, but does it online (not queued)
	def renderTimeframeOnline(self, key, data):
		(year, month, day, category) = key
		req = self._getDummyRequest(datespec=(year, month, day))
		onhome = 0
		toolname = self.name
		if category == '':
			onhome = 1
		body = req.renderPage('BlogmarkRendering',
			year=year,
			month=month,
			day=day,
			category=category,
			onhome=onhome
		)
		html = req.renderPage('BaseRendering',
			body=body,
			year=year,
			month=month,
			day=day,
			category=category,
			onhome=onhome
		)
		fn = self.storeHTML(html, year=year, month=month, day=day, category=category, toolname=toolname)
		if not(year) and not(month) and not(day) and self.rss:
			xml = req.renderPage('RSSFeedRendering', category=category)
			self.storeXML(xml, category=category, toolname=toolname)
		return fn

	# this method returns the title for RSS rendering.
	def getRSSTitle(self, category=''):
		title = self.getPref('title')
		macros = self.getToolByNamespace('macros')
		if not(title):
			prefs = self.getPreferencesTool()
			title = prefs.get('private', 'title')
		if category:
			cat = self.getCategory(category)
			if cat and cat['title']:
				title = cat['title']
		if title:
			return macros.quoteXML(title)
		else:
			return macros.quoteXML(_('An Unconfigured Python Desktop Server Blogmark'))
			
	# this method returns the description for RSS rendering.
	def getRSSDescription(self, category=''):
		macros = self.getToolByNamespace('macros')
		desc = self.getPref('desc')
		if desc: return macros.quoteXML(desc)
		prefs = self.getPreferencesTool()
		if category:
			cat = self.getCategory(category)
			if cat and cat['desc']:
				return macros.quoteXML(cat['desc'])
			else:
				return macros.quoteXML(prefs.get('private','description'))
		else:
			return macros.quoteXML(prefs.get('private','description'))
	
	def _translateItemToRSS(self, item, category):
		macros = self.getToolByNamespace('macros')
		el = {
			'title': macros.quoteXML(item['title']),
			'link': macros.quoteXML(item['link']),
			'pubdate': macros.TodayFull(now=item['pubtime']),
			'source': '',
			'sourceurl': '',
			'categories': item['categories'],
			'comments': '',
			'trackback': ''
		}
		if category == None:
			el['guid'] = '%s@%s' % (item['id'], self.name)
		else:
			el['guid'] = self.getCloudUrlForItem(item['pubtime'], item['id'], category=category)
		excerpt = item['text']
		excerpt = macros.safeHTML(excerpt)
		el['description'] = macros.quoteXML(excerpt)
		return el
	
	# this method returns an item for RSS rendering. This
	# uses a format for items with different property names.
	# fullrss is ignored, as blogmarks are very short and so
	# are allways delivered with full text.
	def getRSSItem(self, id, category='', fullrss=None):
		post = self.getPost(id)
		if post:
			if category and not(category in post['categories']):
				category = ''
			if not(category):
				if not(post['onhome']):
					if len(post['categories']):
						category = post['categories'][0]
					else:
						category = None
			return self._translateItemToRSS(post, category)
		else:
			return None

	# this method returns the items for RSS rendering. This
	# is much like getRecentPosts, but has a different maxnumbers
	# and uses different property names.
	def getRSSItems(self, category=''):
		items = None
		if category == '':
			items = self.getRecentPosts(maxnumber=50, onhome=1, category='')
		else:
			items = self.getRecentPosts(maxnumber=50, onhome=0, category=category)
		res = []
		for item in items:
			el = self._translateItemToRSS(item, category)
			res.append(el)
		return res
	
	# this method returns a list of hours (GMT) where there are no postings
	def getRSSSkipHours(self, category=''):
		items = []
		if category == '':
			items = self.getRecentPosts(maxnumber=100, onhome=1, category='')
		else:
			items = self.getRecentPosts(maxnumber=100, onhome=0, category=category)
		hours = {}
		for item in items:
			hour = time.gmtime(item['pubtime'])[3]
			if hours.has_key(hour):
				hours[hour] += 1
			else:
				hours[hour] = 1
		nonhours = []
		for hour in range(24):
			if not(hours.has_key(hour)):
				nonhours.append(hour)
			elif hours[hour] < 2:
				nonhours.append(hour)
		return nonhours

	# this method returns a preference setting
	def getPref(self, pref):
		return getattr(self.prefs[0], pref, '')
	
	# this method returns a bookmarklet to blogmark links
	def bookmarklet(self, width=600, height=350, location=0, resizable=1, scrollbars=1, status=0, toolbar=0, directories=0):
		javascript = "javascript:void(window.open('"
		if _flet.remote_access:
			if self.pathprefix() != '/':
				javascript += 'http://%s%s' % (
					_PyDS.remotehostname,
					self.getUrl('new_html')
				)
			else:
				javascript += 'http://%s:%s%s' % (
					_PyDS.remotehostname,
					_PyDS.remoteport,
					self.getUrl('new_html')
				)
		else:
			javascript += 'http://%s:%s%s' % (
				_PyDS.serverhostname,
				_PyDS.serverport,
				self.getUrl('new_html')
			)
		javascript += "?ptitle='+escape(document.title.replace(/%20/g,'+'))"
		javascript += "+'&plink='+escape(top.location.href),'%s'," % _('Blogmark')
		javascript += "'width=%d,height=%d,location=%d,resizable=%d,scrollbars=%d,status=%d,toolbar=%d,directories=%d'" % (
			width, height, location, resizable, scrollbars, status, toolbar, directories
		)
		javascript += "))"
		return '<a href="%s" title="%s">%s</a>' % (javascript, _('drag this to your toolbar'), _('Blogmark This!'))

	# this method builds a list of blogmarks for inclusion into the weblog pages
	def linkListBox(self, request=None, category=None, maxnumber=15):
		nuggets = self.getToolByNamespace('nuggets')
		liste = []
		onhome = 0
		if category == '': onhome = 1
		if request:
			if request.dateGiven and request.dayGiven:
				liste = self.getRecentPosts(year=request.year, month=request.month, day=request.day, onhome=onhome, category=category, maxnumber=maxnumber)
			elif request.dateGiven and not(request.dayGiven):
				liste = self.getRecentPosts(year=request.year, month=request.month, onhome=onhome, category=category, maxnumber=maxnumber)
			else:
				liste = self.getRecentPosts(onhome=onhome, category=category, maxnumber=maxnumber)
		else:
			liste = self.getRecentPosts(onhome=onhome, category=category, maxnumber=maxnumber)
		if len(liste):
			return nuggets.linkListBox(linklist=liste)
		else:
			return ''

__desc__ = _("This tool allows to edit blogmark postings.")

PyDS.Tool.registerTool(BlogmarkTool, 'blogmark', _('Blogmark'), __desc__, 917)

