"""

Python Desktop Server - command queues handle commands in memory

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: CommandQueue.py,v 1.1 2003/09/29 12:14:55 gb Exp $

import threading

# this class implements a command queue for a tool. A command queue handles
# commands that might stump out other commands and are unique (identical
# commands are not executed twice in one run). A command consists of the
# command name, a key for the command to recognize identical commands and
# some data. key and data are whatever you like, but the key must be
# hashable, as it is used as a key to a hash.
class CommandQueue:

	def __init__(self):
		self.hash = {}
		self.list = []
		self.lock = threading.RLock()
	
	def __len__(self):
		return self.length()
	
	def append(self, cmd, key, data):
		try:
			self.lock.acquire()
			if self.hash.has_key(key):
				idx = self.hash[key]
				el = self.list[idx]
				el[0] = cmd
				el[2] = data
			else:
				self.list.append([cmd, key, data])
				self.hash[key] = len(self.list)-1
		finally: self.lock.release()
	
	def length(self):
		try:
			self.lock.acquire()
			return len(self.list)
		finally: self.lock.release()

	def shift(self):
		try:
			self.lock.acquire()
			if len(self.list):
				first = self.list[0]
				del self.list[0]
				del self.hash[first[1]]
				for k in self.hash.keys():
					self.hash[k] -= 1
				return first
			else:
				return None
		finally: self.lock.release()

