"""

Python Desktop Server - Default Configuration

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: DefaultConfig.py,v 1.33 2004/09/26 09:11:22 gb Exp $

import os
import sys
import copy
import threading
from distutils.sysconfig import PREFIX

# this method constructs pathnames and creates those directories
# if they don't already exist
def constructPath(*args):
	path = apply(os.path.join, args)
	try: os.makedirs(path)
	except: pass
	return path

# These variables capture the state of the ConfigLoader
_MainThread = threading.currentThread()
_DataDict = {_MainThread: {}}

# This class encapsulates the whole configuration business. There
# is usually only one instance of it, but data returned might be
# different on a per thread view
class Configuration:

	def __init__(self, prefix, homedir):
		self.setPathVars(prefix, homedir)
		self.setPidFileVar()
	
	def setPathVars(self, prefix, homedir):
		self.PREFIX = prefix
		self.HOMEDIR = homedir
		self.SHAREDDIR = constructPath(self.PREFIX, 'share', 'pyds')
		self.MASTERTOOLDIR = constructPath(self.SHAREDDIR, 'tools')
		self.ROOTDIR = constructPath(self.HOMEDIR, 'www')
		self.STATICDIR = constructPath(self.ROOTDIR, 'static')
		self.IMAGESDIR = constructPath(self.ROOTDIR, 'images')
		self.LOGDIR = constructPath(self.HOMEDIR, 'log')
		self.VARDIR = constructPath(self.HOMEDIR, 'var')
		self.ETCDIR = constructPath(self.HOMEDIR, 'etc')
		self.TOOLDIR = constructPath(self.HOMEDIR, 'tools')
		self.TMPLDIR = constructPath(self.HOMEDIR, 'templates')
		self.FONTDIR = constructPath(self.SHAREDDIR, 'fonts')

	def setPidFileVar(self):
		self.PIDFILE = os.path.join(self.VARDIR, 'pyds.pid')

	def __getattr__(self, attr):
		try:
			return self.__getitem__(attr)
		except KeyError:
			raise AttributeError(attr)
	
	def __getitem__(self, key):
		thread = threading.currentThread()
		dict = _DataDict.get(thread, _DataDict[_MainThread])
		return dict[key]
	
	def __setattr__(self, attr, value):
		thread = threading.currentThread()
		if not(_DataDict.has_key(thread)):
			_DataDict[thread] = copy.copy(_DataDict[_MainThread])
		_DataDict[thread][attr] = value
	
	def __setitem__(self, key, value):
		self.__setattr__(key, value)
	
	def __delattr__(self, attr):
		thread = threading.currentThread()
		del _DataDict[thread][attr]
	
	def __delitem__(self, key):
		self.__delattr__(key)
	
	def has_key(self, key):
		thread = threading.currentThread()
		if _DataDict.has_key(thread):
			return _DataDict[thread].has_key(key)
		else: return 0

# compute some special filenames and paths
try: 
	HOMEDIR = os.environ['PYDS_HOME']
except KeyError: 
	if sys.platform == 'win32': 
		try: 
			home = os.environ['HOME'] # for custom environments
		except KeyError: # more likely default NT/2K/XP install
			home = os.environ['HOMEDRIVE'] + os.environ['HOMEPATH']
	else: 
		home = os.environ['HOME']
	HOMEDIR = constructPath(home, '.PyDS')

# set up the config dispatcher object
config = Configuration(PREFIX, HOMEDIR)

# the titletag for the desktop. Usually not changed
config.titletag = 'PyDS'

# how is your server called?
config.serverhostname = 'localhost'
config.serverip = '127.0.0.1'
config.serverport = 4334

# should the monitor server be started? If yes, set monitorport to
# something different than 0 and add a monitor password.
# Monitor will only run on serverip, not on remoteip!
config.monitorport = 0
config.monitorpassword = ''

# how is your server called when called from remote sites?
# these settings must be overwritten to enable remote accessability
config.remotehostname = ''
config.remoteip = ''
config.remoteport = 4334

# this is the default language
config.language = 'en'

# character encoding used for document data
config.documentEncoding = 'iso-8859-1'

# the prefix of your PyDS installation. This can be changed if you
# run your PyDS behind an Apache with a Directory directive. It must
# end with / and is only used on access via the remoteip server!
config.pathprefix = '/'

# the first week day for calendar rendering.
# Monday = 0, Tuesday = 1, ... and Sunday = 6
config.firstweekday = 0

# this is the timeout for socket operations. You might
# need to set this to some higher value if you get many
# timeout errors on upstreaming.
config.sockettimeout = 60

# these usually are't overwritten, they are used by Server.py and restore
config.echoLog = 0
config.proxy = ''
config.verbose = 0
config.verboseLocking = 0
config.daemon = 1

