"""

Python Desktop Server - handle enclosures

Copyright (c) 2003, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: EnclosureTool.py,v 1.12 2004/02/04 09:56:39 gb Exp $

import re
import time
import urllib

import PyDS.Tool

from PyDS.PersistentQueue import PersistentQueue

class EnclosureTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("enclosures[guid:S,url:S,mimetype:S,size:I]")
		db.getas("prefs[active:I,starthour:I,endhour:I]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.enclosures = self.db.view('enclosures').ordered(1)
			self.prefs = self.db.view('prefs')
			if len(self.prefs) == 0:
				self.prefs.append({
					'starthour':23,
					'endhour':6
				})
			self.queue = PersistentQueue(self.name, 'download')
			self._commit()
		finally: self._release()
		
	def _initactive(self):
		weblog = self.getToolByNamespace('weblog')
		weblog._addHook('edit', 'select', _('Enclosure'), self._enclosureForm, self._enclosuresActive)
		weblog._addHook('editold', 'enclosure', '', self._enclosureAdd, self._enclosuresActive)
		weblog._addHook('postnew', 'enclosure', '', self._enclosureAdd, self._enclosuresActive)
	
	def _backup(self):
		liste = self._backupTable('enclosures')
		liste.extend(self._backupTable('prefs'))
		return liste

	def _isRightTime(self):
		now = time.time()
		hour = time.localtime(time.time())[3]
		start = self.getPref('starthour')
		end = self.getPref('endhour')
		if start <= end:
			return (start <= hour) and (end >= hour)
		else:
			return (hour >= start) or (hour <= end)

	def _initthread(self, server):
		pref = self.getPreferencesTool()
		while server.isRunning():
			time.sleep(10)
			if self._isRightTime() and self.queue.length() and pref.online():
				while self.queue.length():
					(cmd, key, data) = self.queue.shift()
					jobf = getattr(self, cmd)
					jobf(key, data)
				self.logVerbose(_('work done, going to sleep again'))

	def _enclosuresActive(self, *args, **kw):
		return self.getPref('active')
	
	def _enclosureForm(self, req, *args, **kw):
		pid = req.getFirstValue('pid')
		val = ''
		if pid:
			weblog = self.getToolByNamespace('weblog')
			item = weblog.getRSSItem(pid)
			val = self.getEnclosureForGUID(item['guid']).get('url', '')
		html = '<input name="encurl" size=40 value="%s">' % val
		return html
	
	def _enclosureAdd(self, tool, req, id, *args, **kw):
		encurl = req.getFirstValue('encurl')
		if encurl:
			item = tool.getRSSItem(id)
			(encurl, mimetype, size) = self.addEnclosureForGUID(item['guid'], encurl)
			for cat in item['categories']:
				itm = tool.getRSSItem(id, category=cat)
				self.addEnclosureForGUID(itm['guid'], encurl, mimetype, size)
	
	def _enclosureFetch(self, key, data):
		downstream = self.getToolByNamespace('downstream')
		try:
			downstream.download(key)
		except IOError:
			self.logError(_('error when fetching <a href="%s">%s</a>') % (key, key))

	def getPreferencePanels(self):
		return [
			['enclosure_panel', _('Enclosures'),
				_('Define the behaviour of the enclosure handler.')]
		]
			

	def enclosure_panel(self, req):
		macros = self.getToolByNamespace('macros')
		html = _('<p>How and when enclosures are fetched or sent')
		html += '<form action="%s" method="POST">' % self.getUrl('enclosure_redir')
		html += '<table border=0 width="100%">'
		html += '<tr><td>%s</td><td>' % _('start hour')
		html += '<input name="starthour" size=3 value="%s">' % self.getPref('starthour')
		html += '</td></tr><tr><td>%s</td><td>' % _('end hour')
		html += '<input name="endhour" size=3 value="%s">' % self.getPref('endhour')
		html += '</td></tr><tr><td>%s</td><td>' % _('send enclosures?')
		html += macros.checkBox('active', self.getPref('active'))
		html += '</table>'
		html += '<input type=submit name="EnclosureSubmit" value="%s">' % _('do it')
		html += '</form>'
		return html
	
	def enclosure_redir(self, req):
		try:
			self._acquire()
			self.prefs[0].active = int(req.getFirstValue('active', 0))
			self.prefs[0].starthour = int(req.getFirstValue('starthour',23))
			self.prefs[0].endhour = int(req.getFirstValue('endhour',6))
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='enclosure_panel',
				fortool='enclosures'
			)
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d enclosures still to be downloaded</li>') % self.queue.length()
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
	def downloadEnclosure(self, url):
		downstream = self.getToolByNamespace('downstream')
		if not(downstream.isCacheReady(url)):
			self.log(_('New enclosure at <a href="%s">%s</a>'), url, url)
			self.queue.append('_enclosureFetch', url, url)
		else:
			self.logVerbose(_('Old enclosure at <a href="%s">%s</a> - no download needed'), url, url)
		return self.queue.length()

	def addEnclosureForGUID(self, guid, enclosure, mimetype=None, size=None):
		try:
			self._acquire()
			downstream = self.getToolByNamespace('downstream')
			opener = downstream.getUrlOpener(0)
			if not(mimetype) or not(size):
				try:
					handle = opener.open(enclosure)
					size = int(handle.info().get('Content-Length'))
					mimetype = handle.info().get('Content-Type', 'application/octetstream')
				except IOError:
					pass
			if size and mimetype:
				self.enclosures.append({
					'guid':guid,
					'url':enclosure,
					'mimetype':mimetype,
					'size':size
				})
				self._commit()
				return (enclosure, mimetype, size)
			else:
				return ()
		finally: self._release()
	
	def getEnclosureForGUID(self, guid):
		try:
			self._acquire()
			(idx, found) = self.enclosures.locate({'guid':guid})
			if found:
				return {
					'url':self.enclosures[idx].url,
					'type':self.enclosures[idx].mimetype,
					'length':self.enclosures[idx].size
				}
			else:
				return {}
		finally: self._release()
	
	def getPref(self, pref):
		try:
			self._acquire()
			if hasattr(self.prefs[0], pref):
				return getattr(self.prefs[0], pref)
			else:
				return ''
		finally: self._release()
	
__desc__ = _("This tool handles enclosures both in aggregator and weblog.")

PyDS.Tool.registerTool(EnclosureTool, 'enclosures', _('Enclosures'), __desc__, 1311)

