"""

Python Desktop Server - External Stories Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: ExternalStoryTool.py,v 1.26 2004/04/26 18:15:04 gb Exp $

import os
import time
import stat
import PyDS.Tool

class ExternalStoryTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("stories[id:S,pubtime:I,title:S,desc:S,structured:I,publish:I,renderhook:S]")
		db.getas("prefs[path:S,title:S,desc:S,interval:I]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.stories = self.db.view('stories').ordered(1)
			self.prefs = self.db.view('prefs')
			if len(self.prefs) == 0:
				self.prefs.append({'interval':60})
			self._commit()
		finally: self._release()
	
	def _backup(self):
		l = self._backupTable('prefs')
		l.extend(self._backupTable('stories', sortcols=['id']))
		return l

	def _initopts(self):
		self.rss = 1

	def _initactive(self):
		navigation = self.getToolByNamespace('navigation')
		navigation._addHook('select', 'estoryindex',
			_('Link to the external story index'),
			self._storyIndexNavHook,
			self.hasStories)
		navigation._addHook('select', 'estory',
			_('Link to an external story'),
			self._storyNavHook,
			self.hasStories)

	def _resync(self, theMirror):
		for story in self.stories:
			if story.publish:
				theMirror.iHave(self, story.id)

	def _storyIndexNavHook(self, qual, value, preset=None):
		if qual == 'html':
			return ''
		elif qual == 'url':
			return self.getStaticUrl()
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _storyNavHook(self, qual, value, preset=None):
		if qual == 'html':
			html = '<select name="%s">' % value
			for story in self.getStories():
				selected = ''
				if story['id'] == preset:
					selected = ' selected'
				html += '<option value="%s"%s>%s' % (
					story['id'],
					selected,
					story['title']
				)
			html += '</select>'
			return html
		elif qual == 'url':
			return self.getStoryUrl(value)
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _initthread(self, server):
		now = time.time()
		nextCheck = now
		while server.isRunning():
			time.sleep(10)
			now += 10
			if now > nextCheck:
				self.logVerbose(_('checking for filesystem changes'))
				files = {}
				for id in self.getIncomingIDs():
					files[id] = 1
				stories = {}
				for id in self.getStoryIDs():
					if not(files.has_key(id)):
						self.deleteStory(id)
				for id in files.keys():
					self.addStory(id)
				nextCheck += self.getPref('interval')
			if self.queue.length():
				while self.queue.length():
					(jobf, key, data) = self.queue.shift()
					jobf(key, data)
				self.logVerbose(_('work done, going to sleep again'))

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d stories in the database</li>') % len(self.stories)
			html += _('<li>There are %d stories in the render queue</li>') % self.queue.length()
			html += '</ul>'
			return html
		finally: self._release()

	# this method returns the description of a page for rendering
	def getPageDescription(self, context):
		desc = PyDS.Tool.StandardTool.getPageDescription(self, context)
		if self.getPref('desc'):
			desc = self.getPref('desc')
		story = context.get('story')
		if story and story['desc']:
			desc = story['desc']
		return desc
		
	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = ''
		html += "<ul>"
		try:
			self._acquire()
			for story in self.stories:
				html += '<li><b><a href="%s">%s (%s)</a>:</b> %s <span class="smallfont">(%s %s)</span></li>' % (
					self.getUrl('show_html', id=str(story.id)),
					story.title,
					story.id,
					story.desc,
					_('last change'),
					macros.Today(story.pubtime, withtime=1),
				)
		finally: self._release()
		html += "</ul>"
		req.setLocalValue('title', _("Your external Stories"))
		return req.renderPage('BaseTemplate', body=html)
		
	def show_html(self, req):
		macros = self.getToolByNamespace('macros')
		sid = req.getFirstValue('id')
		story = self.getStory(sid)
		story['rendered'] = '<pre>%s</pre>' % macros.quoteHTML(self.getStorySource(sid))
		story['staticurl'] = self.getStaticUrl(page=story['pageid'])
		html = req.renderPage(
			'StoryTemplate',
			story=story
		)
		req.setLocalValue('title', story['title'])
		return req.renderPage('BaseTemplate', body=html)
		
	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		story = self.getStory(id)
		title = ''
		text = ''
		desc = ''
		publish = 0
		renderhook = ''
		structured = self.getPreferencesTool().get('private','structured')
		if story:
			title = story['title']
			text = self.getStorySource(id)
			desc = story['desc']
			structured = story['structured']
			publish = story['publish']
			renderhook = story['renderhook']
		html = ''
		html += '<form action="%s" method="POST">' % self.getUrl('edit_redir')
		if story:
			html += '<input type="hidden" name="id" value="%s">' % id
		html += '<table border=0>'
		html += '<tr>'
		if story:
			html += '<th align="right">%s</th><td>%s</td>' % (_('Story'), id)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><input name="title" value="%s" size=50></td>' % (
			_('Title'), title
		)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Publish')
		html += '<select name="publish">'
		if publish:
			html += '<option value="1" selected>%s' % _('yes')
			html += '<option value="0">%s' % _('no')
		else:
			html += '<option value="1">%s' % _('yes')
			html += '<option value="0" selected>%s' % _('no')
		html += '</select>'
		html += '</td>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Textformat')
		html += '<select name="structured">'
		if structured:
			html += '<option value="1" selected>%s' % _('structured text')
			html += '<option value="0">%s' % _('HTML source')
		else:
			html += '<option value="1">%s' % _('structured text')
			html += '<option value="0" selected>%s' % _('HTML source')
		html += '</select></td>'
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('run on rendering')
		html += '<select name="renderhook">'
		html += '<option value="">%s' % _('no macro selected')
		for macro in macros._getMacros():
			if renderhook == macro['name']:
				html += '<option value="%s" selected>%s' % (
					macro['name'],
					macro['name']
				)
			else:
				html += '<option value="%s">%s' % (
					macro['name'],
					macro['name']
				)
		html += '</select></td>'
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="desc" rows=5 cols=60>%s</textarea></td>' % (
			_('Description'), macros.quoteHTML(desc)
		)
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><pre class="whiteboxsmall">%s</pre></td>' % (
			_('Text'), text
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="StorySave" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different story')
		)
		req.setLocalValue('title', _('Edit your story below'))
		return req.renderPage('BaseTemplate', body=html)
		
	def edit_redir(self, req):
		id = req.getFirstValue('id')
		title = req.getFirstValue('title')
		desc = req.getFirstValue('desc')
		structured = int(req.getFirstValue('structured'))
		publish = int(req.getFirstValue('publish'))
		renderhook = req.getFirstValue('renderhook')
		if title:
			self.addStory(id, title=title, desc=desc, structured=structured, publish=publish, renderhook=renderhook, force=1)
			return self.getUrl('show_html', id=id)
		else:
			if id:
				return self.getUrl('edit_html', id=str(id), errmsg=_('You need to give a title!'))
			else:
				return self.getUrl('edit_html', errmsg=_('You need to give a title!'))

	def getPreferencePanels(self):
		return [
			['stories_panel', _('External Stories'),
				_('Here you can configure the title and description of the external stories index page.')]
		]
			
	def stories_panel(self, req):
		macros = self.getToolByNamespace('macros')
		html = '<p>%s' % _('Here you can configure the title and description of the external stories index page and the incoming path.')
		html += '<form action="%s" method="POST">' % req.getUrl('stories_redir', tool='externals')
		html += '<table border=0 width="100%">'
		html += '<tr><td>%s</td><td>' %_('title')
		html += '<input name="title" size=50 value="%s">' % self.getPref('title')
		html += '</td></tr><tr><td>%s</td><td>' % _('description')
		html += '<textarea name="desc" rows=5 cols=50>%s</textarea>' % macros.quoteHTML(self.getPref('desc'))
		html += '</td></tr><tr><td>%s</td><td>' % _('incoming path')
		html += '<input name="path" size=50 value="%s">' % self.getPref('path')
		html += '</td></tr><tr><td>%s</td><td>' % _('check interval')
		html += '<input name="interval" size=5 value="%d">' % self.getPref('interval')
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="StoriesSubmit" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		return html
		
	def stories_redir(self, req):
		title = req.getFirstValue('title')
		desc = req.getFirstValue('desc')
		path = req.getFirstValue('path')
		interval = int(req.getFirstValue('interval'))
		if interval < 60:
			interval = 60
		try:
			self._acquire()
			self.prefs[0].title = title
			self.prefs[0].desc = desc
			self.prefs[0].path = path
			self.prefs[0].interval = interval
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='stories_panel',
				fortool='externals'
			)
		finally: self._release()
		
	def _renderStoryOnline(self, id, dummy):
		macros = self.getToolByNamespace('macros')
		mirror = self.getToolByNamespace('mirror')
		story = self.getStory(id)
		if story and story['publish']:
			req = self._getDummyRequest(now=story['pubtime'])
			req.setLocalValue('title', story['title'])
			story['rendered'] = self.getStoryRendered(id)
			html = req.renderPage(
				'StoryRendering',
				story=story
			)
			src = req.renderPage(
				'BaseRendering',
				story = story,
				body = html,
				category = ''
			)
			self.storeHTML(src, page=story['pageid'])
			mirror.syncItem(self, id)

	# this method puts a render request into the queue
	def renderStory(self, id):
		story = self.getStory(id)
		if story and story['publish']:
			self.queue.append(self._renderStoryOnline, id, None)
		else:
			self.queue.append(self._deleteRenderedStory, id, None)
		self.queue.append(self._renderStoryIndexOnline, 'index.html', None)
		return story['publish']

	def _rerender(self):
		for id in self.getStoryIDs():
			self.renderStory(id)

	def _renderStoryIndexOnline(self, id, dummy):
		macros = self.getToolByNamespace('macros')
		req = self._getDummyRequest()
		req.setLocalValue('title', self.getPageTitle({}))
		html = ''
		html += "<ul>"
		for id in self.getStoryIDs():
			story = self.getStory(id)
			if story['publish']:
				html += '<li><a href="%s">%s</a> <span class="smallfont">(%s %s)</span></li>' % (
					self.getStaticUrl(page=story['pageid']),
					story['title'],
					_('last change'),
					macros.Today(story['pubtime'], withtime=1)
				)
		html += "</ul>"
		src = req.renderPage(
			'BaseRendering',
			body = html,
			category = ''
		)
		self.storeHTML(src)
		xml = req.renderPage('RSSFeedRendering', category='')
		self.storeXML(xml, category='')
	
	def _deleteRenderedStory(self, id, delrecord):
		try:
			self._acquire()
			mirror = self.getToolByNamespace('mirror')
			(idx, found) = self.stories.locate({'id':id})
			if found:
				if delrecord:
					self.log(_('deleting story %s'), id)
					self.stories.delete(idx)
					self._commit()
				self.deleteFile(id)
				mirror.deleteItem(self, id)
			return found
		finally: self._release()

	# this method deletes a story from the database
	def deleteStory(self, id):
		try:
			self._acquire()
			self.queue.append(
				self._deleteRenderedStory,
				id, 1
			)
			self.queue.append(
				self._renderStoryIndexOnline,
				'index.html',
				None
			)
		finally: self._release()
	
	# this method adds a story (should exist in incoming) to the database.
	# if the story already exists, it is updated with data from the
	# filesystem.
	def addStory(self, id, title=None, desc=None, structured=None, publish=None, renderhook=None, force=0):
		try:
			self._acquire()
			now = int(time.time())
			pubtime = 0
			changed = 0
			path = os.path.join(self.prefs[0].path, id)
			if os.path.isfile(path):
				st = os.stat(path)
				pubtime = int(st[stat.ST_MTIME])
			(idx, found) = self.stories.locate({'id':id})
			if not(found):
				idx = self.stories.append({
					'id': id,
					'title': _('%s from filesystem') % id,
					'desc': _('%d bytes') % st[stat.ST_SIZE],
					'pubtime': int(st[stat.ST_MTIME]),
					'publish': 0,
					'structured': 0,
					'renderhook': '',
				})
				self._commit()
				changed = 1
				self.log(_('adding story %s from filesystem at position %d'), id, idx)
			story = self.stories[idx]
			if (pubtime > story.pubtime) and (pubtime + 30 < now):
				story.pubtime = pubtime
				changed = 1
				self.logVerbose(_('change for story %s detected, rerendering'), id)
			if title != None:
				story.title = title
			if desc != None:
				story.desc = desc
			if structured != None:
				story.structured = structured
			if publish != None:
				story.publish = publish
			if renderhook != None:
				story.renderhook = renderhook
			self._commit()
			if changed or force:
				self.renderStory(id)
			return id
		finally: self._release()
	
	# this method returns the ids of stories in the database
	def getStoryIDs(self):
		try:
			self._acquire()
			liste = []
			for story in self.stories:
				liste.append(story.id)
			return liste
		finally: self._release()
	
	# this method returns the ids of stories in the filesystem
	def getIncomingIDs(self):
		path = self.getPref('path')
		if path:
			liste = []
			for file in os.listdir(path):
				if file[0] != '.':
					if os.path.isfile(os.path.join(path, file)):
						liste.append(file)
			return liste
		else:
			return []
	
	def _storyHash(self, story):
		pageid = story.id
		if not pageid.endswith('.html'):
			pageid += '.html'
		return {
			'id': story.id,
			'pageid': pageid,
			'title': story.title,
			'pubtime': int(story.pubtime),
			'desc': story.desc,
			'structured': story.structured,
			'publish': story.publish,
			'renderhook': story.renderhook
		}
	
	# this method returns a story as a dictionary
	def getStory(self, id):
		try:
			self._acquire()
			(idx, found) = self.stories.locate({'id':id})
			if found:
				story = self.stories[idx]
				return self._storyHash(story)
			else:
				return {}
		finally: self._release()
	
	# this returns all stories as a list
	def getStories(self):
		try:
			self._acquire()
			liste = []
			for story in self.stories:
				liste.append(self._storyHash(story))
			return liste
		finally: self._release()
	
	# this method returns the story source
	def getStorySource(self, id):
		if self.getPref('path'):
			path = os.path.join(self.getPref('path'), id)
			if os.path.isfile(path):
				f = open(path)
				src = f.read()
				f.close()
				return src
			else:
				return ''
		else:
			return ''
	
	# this method returns the rendered output of a story
	def getStoryRendered(self, id):
		macros = self.getToolByNamespace('macros')
		story = self.getStory(id)
		src = self.getStorySource(id)
		if story['renderhook'] and hasattr(macros, story['renderhook']):
			src = getattr(macros, story['renderhook'])(story, src)
		req = self._getDummyRequest()
		out = req.renderText(
			src,
			structured=story['structured'],
			story=story,
			homeTool=self,
			uniqueId=story['id']
		)
		return out
	
	# this method returns the URL for a story depending on context
	def getStoryUrl(self, id):
		if _flet.desktop:
			return self.getUrl('show_html', id=id)
		else:
			story = self.getStory(id)
			return self.getStaticUrl(page=story['pageid'])
		
	# do we have external stories?
	def hasStories(self):
		try:
			self._acquire()
			return len(self.stories)
		finally: self._release()
	
	# this method returns the title of the RSS feed
	def getRSSTitle(self, category=''):
		macros = self.getToolByNamespace('macros')
		prefs = self.getPreferencesTool()
		title = prefs.get('private', 'title')
		if self.getPref('title'):
			title = '%s: %s' % (title, self.getPref('title'))
		else:
			title = '%s: %s' % (title, _('Stories'))
		if not(title):
			title = _('An Unconfigured Python Desktop Server: Stories')
		return macros.quoteXML(title)
	
	# this method returns the description of the RSS feed
	def getRSSDescription(self, category=''):
		macros = self.getToolByNamespace('macros')
		prefs = self.getPreferencesTool()
		desc = prefs.get('private', 'description')
		if self.getPref('desc'):
			desc = self.getPref('desc')
		return macros.quoteXML(desc)
	
	def _translateStoryToRSS(self, story, fullrss=0):
		macros = self.getToolByNamespace('macros')
		item = {
			'title': macros.quoteXML(story['title']),
			'link': self.getStaticUrl(page=story['pageid']),
			'guid': self.getStaticUrl(page=story['pageid']),
			'pubdate': macros.TodayFull(now=story['pubtime']),
			'pubtime': story['pubtime'],
			'source': '',
			'sourceurl': '',
			'categories': [],
			'comments': '',
			'trackback': ''
		}
		if fullrss:
			story['rendered'] = self.getStoryRendered(story['id'])
			item['description'] = macros.quoteXML(story['rendered'])
		else: item['description'] = macros.quoteXML(story['desc'])
		return item
	
	# this method returns a single story as an RSS item
	def getRSSItem(self, id, category='', fullrss=0):
		story = self.getStory(id)
		return self._translateStoryToRSS(story, fullrss=fullrss)
	
	# this method returns the stories as an item list, orderd by time
	def getRSSItems(self, category=''):
		items = []
		for id in self.getStoryIDs():
			story = self.getStory(id)
			if story['publish']:
				item = self._translateStoryToRSS(story)
				items.append(item)
		items.sort(lambda a,b: -1*cmp(a['pubtime'],b['pubtime']))
		if len(items) > 20:
			items = items[:21]
		for item in items:
			del item['pubtime']
		return items
	
	# this method returns hours without stories
	def getRSSSkipHours(self, category=''):
		hours = {}
		for id in self.getStoryIDs():
			story = self.getStory(id)
			hours[time.gmtime(story['pubtime'])[3]] = 1
		nonhours = []
		for hour in range(24):
			if not(hours.has_key(hour)):
				nonhours.append(hour)
		return nonhours

	# this method returns a preference
	def getPref(self, pref):
		try:
			self._acquire()
			if hasattr(self.prefs[0], pref):
				return getattr(self.prefs[0], pref)
			else:
				return ''
		finally: self._release()


__desc__ = _("This is a tool that manages longer stories that reside in the filesystem.")

PyDS.Tool.registerTool(ExternalStoryTool, 'externals', _('Externals'), __desc__, 840)

