"""

Python Desktop Server - Upstreaming Driver for Local File System

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: LFSDriver.py,v 1.6 2004/01/02 11:07:15 gb Exp $

import os
import time

from PyDS.DriverRegistry import registerDriver, BaseDriver, getDummyResult

class LFSDriver(BaseDriver):

	# this method initializes the database
	def initdb(self):
		tool = self.tool
		tool.driver = tool.db.getas(
			"lfsdriver[path:S]"
		)
		if len(tool.driver) == 0:
			tool.driver.append({})
		tool._commit()

	def _backup(self):
		l = self.tool._backupTable('lfsdriver', exceptcols=['password'])
		return l

	# this method tells the server that upstreaming needs some
	# mandatory preferences.
	def needsStuff(self, req=None):
		tool = self.tool
		if tool.get('driver', 'path') == '' or tool.get('prefs', 'cloudurl') == '':
			if req:
				return tool.panelErrorUrl(
					req,
					_('You need to give your cloud url and your local path')
				)
			else:
				return 1
		else:
			return 0
				
	def upstream_panel(self, req):
		tool = self.tool
		macros = tool.getToolByNamespace('macros')
		html = '<p>%s' % _("Here you can configure your local path you want to copy to.")
		html += '<form action="%s" method="POST">' % req.getUrl('upstream_redir', tool='upstream')
		html += '<table border=0 width="100%">'
		html += '<tr><td>%s</td><td>' % _('proxy server')
		html += '<input name="proxy" size=20 value="%s">' % tool.get('prefs', 'proxy')
		html += '</td></tr><tr><td>%s</td><td>' % _('local path')
		html += '<input name="path" size=30 value="%s">' % tool.get('driver', 'path')
		html += '</td></tr><tr><td>%s</td><td>' % _('cloud url')
		html += '<input name="cloudurl" size=30 value="%s">' % tool.get('prefs', 'cloudurl')
		html += '</td></tr><tr><td>%s</td><td>' % _('upstream enabled')
		html += macros.checkBox('doupstream', tool.get('prefs', 'doupstream'))
		html += '</td></tr><tr><td>%s</td><td>' % _('upstream interval')
		html += '<input name="upstreaminterval" size=5 value="%d">' % tool.get('prefs', 'upstreaminterval')
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="UpstreamSubmit" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		return html
		
	def upstream_redir(self, req):
		tool = self.tool
		path = req.getFirstValue('path')
		cloudurl = req.getFirstValue('cloudurl')
		proxy = req.getFirstValue('proxy')
		tool.set('driver', 'path', path)
		tool.set('prefs', 'cloudurl', cloudurl)
		tool.set('prefs', 'proxy', proxy)
		tool.set('prefs', 'pinginterval', 0)
		try:
			tool.set('prefs', 'upstreaminterval', int(req.getFirstValue('upstreaminterval',900)))
		except: pass
		try:
			doupstream = int(req.getFirstValue('doupstream',0))
		except:
			doupstream = 0
		doupstreamold = tool.get('prefs', 'doupstream')
		tool.set('prefs', 'doupstream', doupstream)
		tool.set('prefs', 'doping', 0)
		tool._initopts()
		if doupstreamold != doupstream and doupstream:
			tool.awakeUpstream()
		tool.set('prefs', 'usernum', '')
		tool.set('prefs', 'password', '')
		self.getInitialResources()
		self.getServerCapabilities()
		self.pingCloud(1)
		return tool.panelInfoUrl(
			req,
			_("System configured")
		)

	# this calls the radioCommunityServer.getInitialResource via XML-RPC
	# and stores the returned data into the database
	def getInitialResources(self):
		tool = self.tool
		tool.set('prefs', 'commentsurl', '')
		tool.set('prefs', 'trackbackurl', '')
		return getDummyResult()

	# this calls the xmlStorageSystem.getServerCapabilities via XML-RPC
	# and stores the returned data into the database
	def getServerCapabilities(self):
		tool = self.tool
		tool.set('prefs', 'cancomment', 0)
		tool.set('prefs', 'cantrackback', 0)
		tool.set('prefs', 'maxbytes', 200*1024*1024)
		tool.set('prefs', 'bytesused', 0)
		tool.set('prefs', 'maxfilesize', 10*1024*1024)
		tool.set('prefs', 'rankingurl', '')
		tool.set('prefs', 'refererurl', '')
		tool.set('prefs', 'mailtourl', '')
		tool.set('prefs', 'updatesurl', '')
		tool.set('prefs', 'webbugurl', '')
		tool.set('prefs', 'canmirrorposts', 0)
		tool.set('prefs', 'hassearch', 0)
		tool.set('prefs', 'urlsearch', '')
		res = getDummyResult(
			community={
				'flCanHostComments': 0,
				'flCanHostTrackback': 0,
				'flCanMirrorPosts': 0,
				'flHasSearch': 0,
				'urlSearch': ''
			},
			yourUpstreamFolderUrl=tool.get('prefs', 'cloudurl'),
			ctBytesInUse=tool.get('prefs', 'ctBytesInUse'),
			maxBytesPerUser=tool.get('prefs', 'maxBytesPerUser'),
			maxFileSize=tool.get('prefs', 'maxFileSize'),
			urlRankingsByPageReads='',
			urlReferers='',
			urlSpamFreeMailto='',
			urlWeblogUpdates='',
			webBugUrl='',
		)
		return res

	# this calls the xmlStorageSystem.ping via XML-RPC
	# and stores the returned data into the database. This tells
	# the server wether you are online or not.
	def pingCloud(self, online=1, category=''):
		tool = self.tool
		prefs = tool.getToolByNamespace('preferences')
		weblog = tool.getToolByNamespace('weblog')
		tool.set('status', 'email', prefs.get('private', 'email'))
		tool.set('status', 'name', prefs.get('private', 'name'))
		tool.set('status', 'organization', '')
		tool.set('status', 'pings', tool.get('status', 'pings') + 1)
		if tool.get('status', 'membersince') == '':
			tool.set('status', 'membersince', time.ctime())
		tool.set('status', 'behindwall', tool.get('prefs', 'proxy') != '')
		tool.set('status', 'url', tool.get('prefs', 'cloudurl'))
		tool.set('status', 'title', tool.get('prefs', 'title'))
		tool.set('status', 'lastping', time.ctime())
		tool.set('status', 'port', _PyDS.serverport)
		res = getDummyResult(
			cloudData={
				'email': prefs.get('private', 'email'),
				'name': prefs.get('private', 'name'),
				'organization': '',
				'ctFileDeletions': tool.get('status', 'deletes'),
				'whenCreated': tool.get('status', 'membersince'),
				'ctAccesses': tool.get('status', 'pings'),
				'serialNumber': tool.get('status', 'serial'),
				'url': tool.get('prefs', 'cloudurl'),
				'flBehindFirewall': tool.get('prefs', 'proxy') != '',
				'weblogTitle': prefs.get('private', 'title'),
				'ctUpstreams': tool.get('status', 'upstreams'),
				'ctSignons': tool.get('status', 'signons'),
				'whenLastAccess': tool.get('status', 'lastping'),
				'whenLastUpstream': tool.get('status', 'lastupstream'),
				'port': _PyDS.serverport,
				'ctBytesUpstreamed': tool.get('prefs', 'bytesupstreamed'),
				'ctBytesInUse': tool.get('prefs', 'bytesinuse'),
				'whenLastSignOff': tool.get('status', 'lastsignoff'),
				'whenLastSignOn': tool.get('status', 'lastsignon'),
				'flSignedOn': 1,
				'usernum': 'local',
			},
			yourUpstreamFolderUrl=tool.get('prefs', 'cloudurl'),
		)
		return res
	
	# this method deletes multiple files on the cloud
	def deleteMultipleFiles(self, files):
		tool = self.tool
		for fn in files:
			fullname = os.path.join(tool.get('driver', 'path'), fn)
			os.unlink(fullname)
			tool.set('status', 'deletes', tool.get('status', 'deletes') + 1)
		for path in files:
			tool.deleteFileStatus(path)
		return getDummyResult()
			
	# this method upstreams files with given contents without keeping track of
	# their status (actually without the need of them to be in the filesystem!)
	def upstreamMultipleFiles(self, files, contents):
		tool = self.tool
		liste = []
		for idx in range(len(files)):
			fn = files[idx]
			fullname = os.path.join(tool.get('driver', 'path'), fn)
			dirname = os.path.dirname(fullname)
			try: os.makedirs(dirname)
			except: pass
			f = open(fullname, 'wb')
			f.write(contents[idx])
			f.close()
			tool.set('status', 'upstreams', tool.get('status', 'upstreams') + 1)
			tool.set('status', 'bytesupstreamed', tool.get('status', 'bytesupstreamed') + len(contents[idx]))
			url = tool.get('prefs', 'cloudurl')
			if url[-1] == os.sep and fn[0] == os.sep:
				url += fn[1:].replace(os.sep, '/')
			elif url[-1] != os.sep and fn[0] != os.sep:
				url += os.sep
				url += fn.replace(os.sep, '/')
			else:
				url += fn.replace(os.sep, '/')
			liste.append(url)
		tool.set('status', 'lastupstream', time.ctime())
		return getDummyResult(urlList=liste)

registerDriver('LFS', _('Local File System'), LFSDriver)

