"""

Python Desktop Server - metaWeblog API

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: MetaWeblogAPI.py,v 1.26 2004/09/12 16:13:24 gb Exp $

import sys
import sha
import time
import xmlrpclib

import PyDS.Tool

class MetaWeblogAPI(PyDS.Tool.StandardTool):

	def _addEnclosure(self, id, struct):
		item = weblog.getRSSItem(id)
		if item and item['guid']:
			if type(struct['enclosure']) == type({}):
				enclosures.addEnclosureForGUID(item['guid'], struct['enclosure']['url'])
			elif type(struct['enclosure']) in (type(''), type(u'')):
				enclosures.addEnclosureForGUID(item['guid'], struct['enclosure'])

	# this method creates a new posting. The blogid is ignored, as we only have
	# one blog. the struct must conform to the RSS 2.0 struct elements. The
	# following elements are used: title, link, description, category,
	# source. Other items are ignored and some might be inferred from the
	# context. The publish flag is used as a synonym for flNotOnHomePage
	# and an empty category list.
	def newPost(self, blogId, username, password, struct, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 2
		if struct.get('description','<p>')[0] == '<':
			structured = 0
		categories = struct.get('categories',[])
		if type(categories) in (type([]), type((1,1))):
			categories = list(categories)
		else:
			categories = [categories]
		if not(publish): categories = []
		id = weblog.postNewEntry(
			struct.get('title',''),
			struct.get('link',''),
			struct.get('description',''),
			struct.get('source',{}).get('name',''),
			struct.get('source',{}).get('url',''),
			onhome=publish and not(struct.get('flNotOnHomePage',0)),
			categories=categories,
			structured=structured)
		return id
	
	# this method edits an old posting. The blogid is ignored, as we only have
	# one blog. The struct must conform to the RSS 2.0 struct elements. The
	# following elements are used: title, link, description, category,
	# source. Other items are ignored and some might be inferred from the
	# context. The publish flag is used as a synonym for flNotOnHomePage
	# and an emptpy category list.
	def editPost(self, postId, username, password, struct, publish):
		postId = str(postId)
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 2
		if struct.get('description','<p>')[0] == '<':
			structured = 0
		categories = struct.get('categories',[])
		if type(categories) in (type([]), type((1,1))):
			categories = list(categories)
		else:
			categories = [categories]
		if not(publish): categories = []
		id = weblog.editOldEntry(
			postId,
			title=struct.get('title',''),
			link=struct.get('link',''),
			text=struct.get('description',''),
			source=struct.get('source',{}).get('name',''),
			sourceurl=struct.get('source',{}).get('url',''),
			onhome=publish and not(struct.get('flNotOnHomePage',0)),
			categories=categories,
			structured=structured)
		return xmlrpclib.Boolean(1)

	def _translateToRSS(self, post):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		enclosures = self.getToolByNamespace('enclosures')
		res = {'postid':post['id']}
		if post['title']:
			res['title'] = post['title']
		if post['link']:
			res['link'] = post['link']
		if post['categories']:
			res['categories'] = post['categories']
		if post['source'] and post['sourceurl']:
			res['source'] = {
				'name': post['source'],
				'url': post['sourceurl']
			}
		res['description'] = post['text']
		res['dateCreated'] = xmlrpclib.DateTime(post['pubtime'])
		res['flNotOnHomePage'] = not(post['onhome'])
		if post['onhome']:
			res['permaLink'] = weblog.getCloudUrlForItem(post['pubtime'], post['id'])
		elif post['categories'] and len(post['categories']):
			res['permaLink'] = weblog.getCloudUrlForItem(post['pubtime'], post['id'], category=post['categories'][0])
		if res['permaLink']:
			enclosure = enclosures.getEnclosureForGUID(res['permaLink'])
			if enclosure:
				res['enclosure'] = enclosure
		return res

	# this method returns a list of recent postings
	def getRecentPosts(self, blogid, username, password, numberOfPosts):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		liste = []
		for post in weblog.getRecentPosts(maxnumber=numberOfPosts):
			liste.append(self._translateToRSS(post))
		return liste
	
	# this method returns an existing posting in the RSS format. It only
	# returns title, link, description, category and source.
	def getPost(self, postid, username, password):
		postid = str(postid)
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		post = weblog.getPost(postid)
		if post:
			return self._translateToRSS(post)
		else:
			raise _('Posting %s not found') % postid

	# this method returns a list of existing categories
	def getCategories(self, blogid, username, password):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		res = {}
		for cat in weblog.getCategories():
			res[cat['name']] = {
				'description': cat['title'],
				'htmlUrl': weblog.getStaticUrl(category=cat['name']),
				'rssUrl': weblog.getStaticUrl(page='rss.xml', category=cat['name']),
			}
		return res

	# this method uploads a new media object - currently only pictures
	# are allowed! Additional struct parameter are title, description
	# and info for the respective fields in the structure and publish
	# to tell wether a picture should be published or not. The image
	# returned is one with a bounding box of 200 pixes in both
	# directions, not the original image!
	def newMediaObject(self, blogid, username, password, struct):
		prefs = self.getPreferencesTool()
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure'), username
		assert struct.get('name', ''), 'required parameter name'
		assert struct.get('type', ''), 'required parameter type'
		assert struct.get('bits', ''), 'required parameter bits'
		(base, ext) = struct['type'].split('/')
		assert base == 'image', 'only images allowed'
		title = struct.get('title', '') or struct['name']
		desc = struct.get('description', '')
		info = struct.get('info', '')
		publish = struct.get('publish', 0)
		fname = sha.new()
		fname.update(title)
		fname.update(desc)
		fname.update(info)
		fname.update(str(time.time()))
		fname = 'upload-%s.%s' % (fname.hexdigest(), ext)
		path = self.getFileName(fname, cloud=0)
		self.storeFile(struct['bits'].data, fname, cloud=0)
		pictures = self.getToolByNamespace('pictures')
		id = pictures.uploadFromPath(title, desc, path, info=info, publish=publish, delete=1)
		return {
			'url': pictures.imageUrl(id, size=(200,200))
		}

__desc__ = _("This tool implements the metaWeblog API.")

PyDS.Tool.registerTool(MetaWeblogAPI, 'metaWeblog', _('metaWeblog API'), __desc__, 1411)

