"""

Python Desktop Server - Navigation Box Editor and Renderer

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: NavigationTool.py,v 1.37 2004/02/04 09:56:39 gb Exp $

import PyDS.Tool
import metakit

navTemplate = """<table class="navigationbox" border="0">
#for $link in $navigation.getLinks()
<tr><td class="$link.class"><a class="navigation-reference" href="$link.url" title="$link.description">$link.title</a></td></tr>
#end for
</table>"""

linktypelist = [
	['home', _('Link to your homepage'), 0],
	['category', _('Link to a weblog category'), 1],
	['story', _('Link to a story'), 1],
	['storyindex', _('Link to the story index'), 0],
	['estory', _('Link to an external story'), 1],
	['estoryindex', _('Link to the external story index'), 0],
	['picture', _('Link to a picture'), 1],
	['pictureindex', _('Link to the picture index'), 0],
	['macroindex', _('Link to the macro index'), 0],
	['nuggetindex', _('Link to the nugget index'), 0],
	['wiki', _('Link to a wiki page'), 0],
	['direct', _('Direct link to some url'), 1],
]
linktypes = {}
for lt in linktypelist:
	linktypes[lt[0]] = lt[1]

class NavigationTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("navigation[level:I,title:S,type:S,ctxtool:S,name:S,desc:S]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.navigation= self.db.view('navigation').ordered(2)
			self.importantTool = 1
		finally: self._release()
	
	def _backup(self):
		return self._backupTable('navigation')

	# this method returns the URL of the tool itself to be used
	# in menus and places like that.
	def url(self):
		return self.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)

	def _initopts(self):
		navigation = self.getToolByNamespace('navigation')
		nuggets = self.getToolByNamespace('nuggets')
		navigation._addHook('select', 'home',
			_('Link to your homepage'),
			self._homeNavHook)
		navigation._addHook('select', 'direct',
			_('Direct link to some url'),
			self._directNavHook)
		nuggets.addStandardNugget('navigationBox',
			_('Template for the navigational menu'),
			navTemplate)
	
	
	def _directNavHook(self, qual, value, preset=None):
		if qual == 'html':
			if preset:
				html = '<input name="%s" size="30" value="%s">' % (value, preset)
			else:
				html = '<input name="%s" size="30">' % value
			return html
		elif qual == 'url':
			return value
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _homeNavHook(self, qual, value, preset=None):
		if qual == 'html':
			return ''
		elif qual == 'url':
			return self.getToolByNamespace('upstream').get('prefs','cloudurl')
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	# this method returns description of preferences panels that are
	# hooked up into preferences by this tool
	def getPreferencePanels(self):
		return [
			['navigation_panel', _('Navigation'),
				_('You can configure your navigation links here.')]
		]
			
	def navigation_panel(self, req):
		try:
			self._acquire()
			html = '<p>%s' % _("Here you can add and delete navigation links for your homepage.")
			html += '<p><a href="%s">%s</a>' % (
				self.getUrl('renumber_redir'),
				_("Renumber all navigational items.")
			)
			html += '<table border=0 width="100%">'
			for link in self.navigation:
				html += '<tr><td class="whitebox">'
				html += '<table border=0 width="100%"><tr><td width="80%">'
				hook = self._getHook(
					'select',
					'navigation.select.%s' % link.type
				)
				if link.name:
					if link.ctxtool:
						html += '<b>%d.</b> <a href="%s">%s(%s - %s, %s):</a> %s' % (
							link.level,
							hook('url', link.name),
							link.title,
							link.type,
							link.name,
							_('only active in %s') % link.ctxtool,
							link.desc
						)
					else:
						html += '<b>%d.</b> <a href="%s">%s(%s - %s):</a> %s' % (
							link.level,
							hook('url', link.name),
							link.title,
							link.type,
							link.name,
							link.desc
						)
				else:
					if link.ctxtool:
						html += '<b>%d.</b> <a href="%s">%s(%s, %s):</a> %s' % (
							link.level,
							hook('url', link.name),
							link.title,
							link.type,
							_('only active in %s') % link.ctxtool,
							link.desc
						)
					else:
						html += '<b>%d.</b> <a href="%s">%s(%s):</a> %s' % (
							link.level,
							hook('url', link.name),
							link.title,
							link.type,
							link.desc
						)
				html += '</td><td width="20%" align="right" valign="top">'
				html += '%s<br>%s</td></tr>' % (
					'<a href="%s">%s</a>' % (
						req.getUrl(
							'panel_html',
							tool='preferences',
							panel='editnavigation_panel',
							fortool='navigation',
							linktitle=link.title
						), _('edit')
					),
					'<a href="%s">%s</a>' % (
						req.getUrl('delnavigation_redir', tool='navigation', linktitle=link.title),
						_('delete')
					)
				)
				html += '</table></td></tr>'
			html += '</td></tr>'
			html += '<tr><td>'
			html += '<form action="%s" method="POST">' % (
				req.getUrl(
					'panel_html',
					tool='preferences',
					panel='addnavigation_panel',
					fortool='navigation'
				)
			)
			html += '<select name="hook">'
			for hook in self._getHooks('select'):
				if hook.isActive():
					html += '<option value="%s">%s' % (
						hook.getId(),
						hook.title
					)
			html += '</select>'
			html += ' <input type="submit" name="add" value="%s">' % _('Add')
			html += '</form>'
			html += '</td></tr>'
			html += '</table>'
			return html
		finally: self._release()
	
	def addnavigation_panel(self, req):
		hookid = req.getFirstValue('hook')
		hook = self._getHook('select', hookid)
		html = '<form action="%s" method="POST">' % (
			req.getUrl('addnavigation_redir', tool='navigation')
		)
		html += '<table border="0">'
		html += '<tr><th>%s</th><td><input name="linklevel" size=3></td></tr>' % _('link level')
		html += '<tr><th>%s</th><td><input name="linktitle" size=10></td></tr>' % _('link title')
		hookhtml = hook('html', 'linkname')
		if hookhtml:
			html += '<tr><th>%s</th><td>' % _('link object')
			html += hookhtml
		html += '</td></tr>'
		html += '<tr><th>%s</th><td>' % _('show in')
		html += '<select name="linktool">'
		html += '<option value="">%s' % _('all tools')
		for tool in PyDS.Tool.getTools():
			if tool.level < 2000:
				html += '<option value="%s">%s' % (tool.name, tool.title)
		html += '</select>'
		html += '</td></tr>'
		html += '<tr><th>%s</th><td><textarea name="linkdesc" cols=50 rows=5></textarea></td></tr>' % _('link desc')
		html += '<tr><td colspan=2 align=right><input name="add" type="submit" value="%s"></td></tr>' % _('Add')
		html += '</table>'
		html += '<input type="hidden" name="hook" value="%s">' % hookid
		html += '<input type="hidden" name="linktype" value="%s">' % hook.tag
		html += '</form>'
		return html
	
	def editnavigation_panel(self, req):
		macros = self.getToolByNamespace('macros')
		linktitle = req.getFirstValue('linktitle')
		nav = self.getNavigation(linktitle)
		hook = self._getHook('select', 'navigation.select.%s' % nav['type'])
		html = '<form action="%s" method="POST">' % (
			req.getUrl('addnavigation_redir', tool='navigation')
		)
		html += '<table border="0">'
		html += '<tr><th>%s</th><td><input name="linklevel" size=3 value="%s"></td></tr>' % (_('link level'), nav['level'])
		html += '<tr><th>%s</th><td><input name="linktitle" size=10 value="%s"></td></tr>' % (_('link title'), nav['title'])
		hookhtml = hook('html', 'linkname', nav['name'])
		if hookhtml:
			html += '<tr><th>%s</th><td>' % _('link object')
			html += hookhtml
		html += '</td></tr>'
		html += '<tr><th>%s</th><td>' % _('show in')
		html += '<select name="linktool">'
		html += '<option value="">%s' % _('all tools')
		for tool in PyDS.Tool.getTools():
			if tool.level < 2000:
				if nav['ctxtool'] == tool.name:
					html += '<option value="%s" selected>%s' % (tool.name, tool.title)
				else:
					html += '<option value="%s">%s' % (tool.name, tool.title)
		html += '</select>'
		html += '</td></tr>'
		html += '<tr><th>%s</th><td><textarea name="linkdesc" cols=50 rows=5>%s</textarea></td></tr>' % (_('link desc'), macros.quoteHTML(nav['desc']))
		html += '<tr><td colspan=2 align=right><input name="add" type="submit" value="%s"></td></tr>' % _('Add')
		html += '</table>'
		html += '<input type="hidden" name="hook" value="%s">' % hook.getId()
		html += '<input type="hidden" name="linktype" value="%s">' % hook.tag
		html += '</form>'
		return html
	
	def addnavigation_redir(self, req):
		hookid = req.getFirstValue('hook')
		hook = self._getHook('select', hookid)
		title = req.getFirstValue('linktitle')
		typ = req.getFirstValue('linktype')
		name = req.getFirstValue('linkname')
		desc = req.getFirstValue('linkdesc')
		level = int(req.getFirstValue('linklevel'))
		ctxtool = req.getFirstValue('linktool')
		self.addNavigation(level, title, typ, ctxtool, name, desc)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
		
	def delnavigation_redir(self, req):
		title = req.getFirstValue('linktitle')
		self.delNavigation(title)
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
	
	def renumber_redir(self, req):
		self.renumberNavigation()
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='navigation_panel',
			fortool='navigation'
		)
	
	# add or change an existing navigation link
	def addNavigation(self, level, title, typ, ctxtool, name, desc):
		try:
			self._acquire()
			res = self.navigation.select({'title':title})
			if len(res):
				row = res[0]
				row.name = name
				row.type = typ
				row.ctxtool = ctxtool
				row.desc = desc
				row.level = level
			else:
				self.navigation.append({'title':title, 'type':typ, 'ctxtool':ctxtool, 'name':name, 'desc':desc, 'level':level})
			self._commit()
			return len(res)
		finally: self._release()
		
	# delete an existing navigation link
	def delNavigation(self, title):
		try:
			self._acquire()
			todel = []
			idx = 0
			for nav in self.navigation:
				if nav.title == title:
					todel.append(idx)
				idx += 1
			todel.sort()
			todel.reverse()
			for idx in todel:
				self.navigation.delete(idx)
				self._commit()
			return len(todel)
		finally: self._release()
	
	# get the record of an existing navigation link
	def getNavigation(self, title):
		try:
			self._acquire()
			res = self.navigation.select({'title':title})
			if len(res):
				row = res[0]
				hook = self._getHook(
					'select',
					'navigation.select.%s' % row.type
				)
				res = {
					'title':row.title,
					'type':row.type,
					'ctxtool':row.ctxtool,
					'name':row.name,
					'desc':row.desc,
					'level':row.level,
					'url':hook('url', row.name),
				}
				return res
			return {}
		finally: self._release()

	# returns wether any navigation links are defined
	def isactive(self):
		try:
			self._acquire()
			return len(self.navigation) > 0
		finally: self._release()

	# renumbers all navigation items to steps of 10
	def renumberNavigation(self, step=10):
		try:
			self._acquire()
			liste = []
			for nav in self.navigation:
				liste.append(nav.title)
			l = 0
			for t in liste:
				idx = self.navigation.find(title=t)
				if idx >= 0:
					self.navigation[idx].level = l
				l += 10
			self._commit()
		finally: self._release()

	# this method returns a list of links as {title, description, url, context, class} hashes.
	# It uses the dynamic context to find context sensitive links, so you should call it
	# only in templates with correct settings for current!
	def getLinks(self):
		links = []
		for link in self.navigation:
			hook = self._getHook(
				'select',
				'navigation.select.%s' % link.type
			)
			if link.ctxtool == '' or link.ctxtool == _flet.current.name:
				item = {
					'title':link.title,
					'description':link.desc,
					'url':hook('url', link.name),
					'context':link.ctxtool
				}
				if link.ctxtool: item['class'] = 'subnavigationelement'
				else: item['class'] = 'navigationelement'
				links.append(item)
		return links

	# renders the navigation block as HTML
	def render(self):
		nuggets = self.getToolByNamespace('nuggets')
		return nuggets.navigationBox()

__desc__ = _("This tool allows to configure navigation links.")

PyDS.Tool.registerTool(NavigationTool, 'navigation', _('Navigation'), __desc__, 1100)

