"""

Python Desktop Server - persistent queues handle commands in metakit

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: PersistentQueue.py,v 1.3 2003/09/29 15:28:03 gb Exp $

import threading
import metakit
import os

try:
	from cPickle import dumps, loads
except ImportError:
	from pickle import dumps, loads

# this class implements a command queue for a tool. A command queue handles
# commands that might stump out other commands and are unique (identical
# commands are not executed twice in one run). A command consists of the
# command name, a key for the command to recognize identical commands and
# some data. key and data are whatever you like, but they must be pickable,
# as they are stored as pickled strings in metakit
class PersistentQueue:

	def __init__(self, toolname, queuename):
		self.lock = threading.RLock()
		try:
			self.lock.acquire()
			self.store = metakit.storage(
				os.path.join(_PyDS.VARDIR, '%s.queue' % toolname), 1)
			self.queue = self.store.getas(
				"%s[key:S,cmd:S,data:S]" % queuename
			).ordered(1)
		finally: self.lock.release()
	
	def __len__(self):
		return self.length()
	
	def append(self, cmd, key, data):
		try:
			self.lock.acquire()
			key = dumps(key)
			data = dumps(data)
			(idx, found) = self.queue.locate({'key':key})
			if found:
				self.queue[idx].data = data
			else:
				self.queue.append({
					'key':key,
					'cmd':cmd,
					'data':data
				})
			self.store.commit()
		finally: self.lock.release()
	
	def length(self):
		try:
			self.lock.acquire()
			return len(self.queue)
		finally: self.lock.release()

	def shift(self):
		try:
			self.lock.acquire()
			if self.length():
				cmd = self.queue[0].cmd
				key = loads(self.queue[0].key)
				data = loads(self.queue[0].data)
				self.queue.delete(0)
				self.store.commit()
				return (cmd, key, data)
			else:
				return None
		finally: self.lock.release()

