"""

Python Desktop Server - Weblogs Ping Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: PingTool.py,v 1.17 2004/03/17 08:23:17 gb Exp $

import re
import sys
import PyDS.Tool

rssfeedtitlere = re.compile(r'<title>(.*?)</title>')
def getRSSFeedTitle(xml):
	match = rssfeedtitlere.search(xml)
	if match:
		return match.group(1)
	else:
		return PyDS.Tools.getTool('preferences').get('private','title')

rssfeedlinkre = re.compile(r'<link>(.*?)</link>')
def getRSSFeedLink(xml):
	match = rssfeedlinkre.search(xml)
	if match:
		return match.group(1)
	else:
		return None

class PingTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("prefs[weblogscom:I,localcom:I,technorati:I,blogs:I,weblogscomcharset:I,localcomcharset:I,technoraticharset:I,blogscharset:I]")
		db.getas("pings[url:S,type:S,charset:I]")
	
	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.prefs = self.db.view('prefs')
			if len(self.prefs) == 0:
				self.prefs.append({
					"weblogscom":1,
					"weblogscomcharset":0,
					"localcom":1,
					"localcomcharset":0,
					"technorati":1,
					"technoraticharset":1,
					"blogs":1,
					"blogscharset":1
				})
			self.pings = self.db.view('pings').ordered(1)
			self._commit()
			self.importantTool = 1
		finally: self._release()

	def _initopts(self):
		upstream = self.getToolByNamespace('upstream')
		upstream.hookURLSuffix('/rss.xml', 900, self._pingUpdatesForRSS)

	def _pingUpdatesForRSS(self, url, xml):
		macros = self.getToolByNamespace('macros')
		upstream = self.getToolByNamespace('upstream')
		title = getRSSFeedTitle(xml)
		link = getRSSFeedLink(xml)
		if self.getPref('weblogscom'):
			rpcurl = 'http://rpc.weblogs.com/RPC2'
			self.pingUpdates(rpcurl, title, link, no8bit=not(self.getPref('weblogscomcharset')))
			rssrpcurl = 'http://rssrpc.weblogs.com/RPC2'
			self.pingRSSUpdates(rssrpcurl, title, url, no8bit=not(self.getPref('weblogscomcharset')))
		if self.getPref('technorati'):
			rpcurl = 'http://rpc.technorati.com/rpc/ping'
			self.pingExtendedUpdates(rpcurl, title, link, url, no8bit=not(self.getPref('technoraticharset')))
		if self.getPref('blogs'):
			rpcurl = 'http://ping.blo.gs/'
			self.pingExtendedUpdates(rpcurl, title, link, url, no8bit=not(self.getPref('blogscharset')))
		if self.getPref('localcom') and upstream.get('prefs', 'rpcurl'):
			rpcurl = upstream.get('prefs', 'rpcurl')
			self.pingUpdates(rpcurl, title, link, no8bit=not(self.getPref('localcomcharset')))
		for ping in self.getPingServers():
			if ping['type'] == 'ping':
				self.pingUpdates(ping['url'], title, link, no8bit=not(ping['charset']))
			elif ping['type'] == 'extendedPing':
				self.pingExtendedUpdates(ping['url'], title, link, url, no8bit=not(ping['charset']))
			elif ping['type'] == 'rssUpdate':
				self.pingRSSUpdates(ping['url'], title, link, no8bit=not(ping['charset']))
		return []
	
				
	# this method returns descriptions of hooked up preferences panels
	# for this tool
	def getPreferencePanels(self):
		return [
			['ping_panel', _('Ping Updates'),
				_('This panel configures the central servers that are notified on changed RSS feeds.')]
		]
	
	def ping_panel(self, req):
		macros = self.getToolByNamespace('macros')
		try:
			self._acquire()
			html = '<p>%s' % _('This panel configures the central servers that are notified on changed RSS feeds.')
			html += '<form action="%s" method="POST">' % req.getUrl('ping_redir', tool='ping')
			html += '<table border=0 width="100%">'
			html += '<tr><th align="left">%s</th><th align="left">%s</th><th align="left">%s</th></tr>' % (
				_('Server'),
				_('active'),
				_('utf-8 capable')
			)
			html += '<td>%s</td><td>' % _('weblogs.com')
			html += macros.checkBox('weblogscom', self.prefs[0].weblogscom)
			html += '</td><td>'
			html += macros.checkBox('weblogscomcharset', self.prefs[0].weblogscomcharset)
			html += '</td></tr><tr><td>%s</td><td>' % _('technorati.com')
			html += macros.checkBox('technorati', self.prefs[0].technorati)
			html += '</td><td>'
			html += macros.checkBox('technoraticharset', self.prefs[0].technoraticharset)
			html += '</td></tr><tr><td>%s</td><td>' % _('blo.gs')
			html += macros.checkBox('blogs', self.prefs[0].blogs)
			html += '</td><td>'
			html += macros.checkBox('blogscharset', self.prefs[0].blogscharset)
			html += '</td></tr><tr><td>%s</td><td>' % _('local community server')
			html += macros.checkBox('localcom', self.prefs[0].localcom)
			html += '</td><td>'
			html += macros.checkBox('localcomcharset', self.prefs[0].localcomcharset)
			html += '</td></tr><tr><td colspan=2 align="right">'
			html += '<input type=submit name="UpstreamSubmit" value="%s">' % _('save')
			html += '</td></tr>'
			html += '</table>'
			html += '</form>'
			pingservers = self.getPingServers()
			if pingservers:
				html += _('<p>These are your additional ping servers:')
				html += '<ul>'
				for ping in pingservers:
					html += '<li><a href="%s">%s</a> (%s, %s) - <a href="%s">%s</a></li>' % (
						ping['url'],
						ping['url'],
						ping['type'],
						(ping['charset'] and _('utf-8')) or _('entities'),
						req.getUrl('delping_redir', tool='ping', pingurl=ping['url']),
						_('delete')
					)
				html += '</ul>'
			html += _('<p>Add a user defined ping server here:')
			html += '<p><form action="%s" method="POST">' % req.getUrl('newping_redir', tool='ping')
			html += '<input name="pingurl" size=30>'
			html += ' <select name="pingtype">'
			html += ' <option value="ping">ping'
			html += ' <option value="extendedPing">extendedPing'
			html += ' <option value="rssUpdate">rssUpdate'
			html += ' </select>'
			html += ' <select name="pingcharset">'
			html += ' <option value="0">%s' % _('use entities')
			html += ' <option value="1">%s' % _('utf-8 capable')
			html += ' </select>'
			html += ' <input type=submit name="PingAdd" value="%s">' % _('add')
			html += '</form>'
		finally: self._release()
		return html
		
	def ping_redir(self, req):
		try:
			self._acquire()
			try:
				self.prefs[0].weblogscom = int(req.getFirstValue('weblogscom',1))
				self.prefs[0].weblogscomcharset = int(req.getFirstValue('weblogscomcharset',0))
				self.prefs[0].technorati = int(req.getFirstValue('technorati',1))
				self.prefs[0].technoraticharset = int(req.getFirstValue('technoraticharset',1))
				self.prefs[0].blogs = int(req.getFirstValue('blogs',1))
				self.prefs[0].blogscharset = int(req.getFirstValue('blogscharset',1))
				self.prefs[0].localcom = int(req.getFirstValue('localcom',1))
				self.prefs[0].localcomcharset = int(req.getFirstValue('localcomcharset',0))
			except: pass
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='ping_panel',
				fortool='ping'
			)
		finally: self._release()
	
	def newping_redir(self, req):
		try:
			self._acquire()
			url = req.getFirstValue('pingurl')
			(idx, found) = self.pings.locate({'url':url})
			if not(found):
				self.pings.append({
					'url':req.getFirstValue('pingurl'),
					'type':req.getFirstValue('pingtype'),
					'charset':int(req.getFirstValue('pingcharset',0))
				})
			else:
				self.pings[idx].type = req.getFirstValue('pingtype')
				self.pings[idx].charset = int(req.getFirstValue('pingcharset',0))
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='ping_panel',
				fortool='ping'
			)
		finally: self._release()
	
	def delping_redir(self, req):
		try:
			self._acquire()
			url = req.getFirstValue('pingurl')
			(idx, found) = self.pings.locate({'url':url})
			if found:
				self.pings.delete(idx)
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='ping_panel',
				fortool='ping'
			)
		finally: self._release()
	
	# this method returns additional ping servers
	def getPingServers(self):
		try:
			self._acquire()
			liste = []
			for ping in self.pings:
				liste.append({
					'url':ping.url,
					'type':ping.type,
					'charset':ping.charset
				})
			return liste
		finally: self._release()

	# this method pings a server that implements the weblogs.com
	# ping interface
	def pingUpdates(self, rpcurl, blogname, blogurl, no8bit=0):
		macros = self.getToolByNamespace('macros')
		upstream = self.getToolByNamespace('upstream')
		try:
			self._acquire()
			server = upstream.getServerForXMLRPC(rpcurl)
			if no8bit:
				blogname = macros.no8bit(blogname)
			else:
				blogname = blogname.decode(_PyDS.documentEncoding).encode('utf-8')
			try:
				res = server.weblogUpdates.ping(blogname, blogurl)
				if res.get('flError'):
					self.log(_('pingUpdates to %s gave error: %s'), rpcurl, res['message'])
				else:
					self.log(_('pingUpdates to %s gave message: %s'), rpcurl, res['message'])
				return res.get('flError')
			except:
				(e, d, tb) = sys.exc_info()
				self.log(_('pingUpdates to %s gave an exception: %s: %s'), rpcurl, e, macros.quoteHTML(str(d)))
			return 1
		finally: self._release()
		
	# this method pings a server that implements the blo.gs
	# extended ping interface
	def pingExtendedUpdates(self, rpcurl, blogname, blogurl, rssurl, no8bit=0):
		macros = self.getToolByNamespace('macros')
		upstream = self.getToolByNamespace('upstream')
		try:
			self._acquire()
			server = upstream.getServerForXMLRPC(rpcurl)
			if no8bit:
				blogname = macros.no8bit(blogname)
			else:
				blogname = blogname.decode(_PyDS.documentEncoding).encode('utf-8')
			try:
				res = server.weblogUpdates.extendedPing(blogname, blogurl, rssurl, rssurl)
				if res.get('flError'):
					self.log(_('pingExtendedUpdates to %s gave error: %s'), rpcurl, res['message'])
				else:
					self.log(_('pingExtendedUpdates to %s gave message: %s'), rpcurl, res['message'])
				return res.get('flError')
			except:
				(e, d, tb) = sys.exc_info()
				self.log(_('pingExtendedUpdates to %s gave an exception: %s: %s'), rpcurl, e, macros.quoteHTML(str(d)))
			return 1
		finally: self._release()
		
	# this method pings a server that implements the weblogs.com
	# rssUpdate interface
	def pingRSSUpdates(self, rpcurl, blogname, blogurl, no8bit=0):
		macros = self.getToolByNamespace('macros')
		upstream = self.getToolByNamespace('upstream')
		try:
			self._acquire()
			server = upstream.getServerForXMLRPC(rpcurl)
			if no8bit:
				blogname = macros.no8bit(blogname)
			else:
				blogname = blogname.decode(_PyDS.documentEncoding).encode('utf-8')
			try:
				res = server.rssUpdate(blogname, blogurl)
				if res.get('flError'):
					self.log(_('pingRSSUpdates to %s gave error: %s'), rpcurl, res['message'])
				else:
					self.log(_('pingRSSUpdates to %s gave message: %s'), rpcurl, res['message'])
				return res.get('flError')
			except:
				(e, d, tb) = sys.exc_info()
				self.log(_('pingRSSUpdates to %s gave an exception: %s: %s'), rpcurl, e, macros.quoteHTML(str(d)))
			return 1
		finally: self._release()
		
	# this method returns a stored preference value
	def getPref(self, field):
		return getattr(self.prefs[0], field, '')


__desc__ = _("This tool allows to configure the servers you want to notify of changed RSS feeds.")

PyDS.Tool.registerTool(PingTool, 'ping', _('Pinging'), __desc__, 1320)

