"""

Python Desktop Server - Mirror items to the PyCS search engine

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: PyCSSearchMirror.py,v 1.14 2004/01/02 20:19:46 gb Exp $

import metakit
import xmlrpclib
import md5

import PyDS.MirrorRegistry

class PyCSSearchMirror(PyDS.MirrorRegistry.BaseMirror):

	def _signature(self, item):
		signature = md5.md5()
		signature.update(item['title'])
		signature.update(item['description'])
		return signature.hexdigest()

	def canWork(self):
		upstream = self.tool.getToolByNamespace('upstream')
		prefs = self.tool.getPreferencesTool()
		return upstream.get('prefs', 'canmirrorposts') and prefs.get('private', 'driver') in ('XSS', '')

	def _addItem(self, tool, uniqueId, item):
		upstream = self.tool.getToolByNamespace('upstream')
		macros = self.tool.getToolByNamespace('macros')
		upstream.server.xmlStorageSystem.mirrorPosts(
			upstream.get('prefs', 'usernum'),
			upstream.get('prefs', 'password'),
			[{
				'date':xmlrpclib.DateTime(macros.parseTodayFull(item['pubdate'])),
				'postid':'%s@%s' % (uniqueId, tool.name),
				'url':item['guid'],
				'guid':item['guid'],
				'title':macros.unquoteXML(item['title']).decode(_PyDS.documentEncoding),
				'description':macros.unquoteXML(item['description']).decode(_PyDS.documentEncoding)
			}]
		)
	
	def _updateItem(self, tool, uniqueId, item):
		self._addItem(tool, uniqueId, item)
	
	def _deleteItem(self, tool, uniqueId):
		upstream = self.tool.getToolByNamespace('upstream')
		upstream.server.xmlStorageSystem.deleteMirroredPosts(
			upstream.get('prefs', 'usernum'),
			upstream.get('prefs', 'password'),
			['%s@%s' % (uniqueId, tool.name)]
		)

PyDS.MirrorRegistry.registerMirror('pycsSearch', _('This mirror replicates items to the PyCS search engine'), PyCSSearchMirror)

