"""

Python Desktop Server - Shortcut Editor

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: ShortcutTool.py,v 1.26 2004/03/04 12:57:23 gb Exp $

import PyDS.Tool
import PyDS.StructuredText
import metakit
import re
import docutils.transforms

shorties = {
	':-)': "$macros.imageTag('/images/smile.gif',alt=%s)" % _('"Smiley"'),
	';-)': "$macros.imageTag('/images/wink.gif',alt=%s)" % _('"Winky"'),
	':-O': "$macros.imageTag('/images/astonished.gif',alt=%s)" % _('"Astonished"'),
	':-(': "$macros.imageTag('/images/unsmile.gif',alt=%s)" % _('"Unsmiley"'),
	'PyDS': "$macros.linkTag('http://pyds.muensterland.org/', 'Python Desktop Server')",
	'PyCS': "$macros.linkTag('http://www.pycs.net/', 'Python Community Server')",
	'bzero': "$macros.linkTag('http://www.myelin.co.nz/bzero/','bzero')",
	'Radio': "$macros.linkTag('http://radio.userland.com/', 'Radio Userland')",
}

class ShortcutTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("shortcuts[short:S,long:S]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.shortcuts= self.db.view('shortcuts').ordered(1)
			self.importantTool = 1
		finally: self._release()
	
	def _backup(self):
		return self._backupTable('shortcuts')
		
	def _initopts(self):
		try:
			self._acquire()
			self.shortcutcache = {}
			for s in self.shortcuts:
				self.shortcutcache[s.short] = s.long
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d shortcuts defined</li>') % len(self.shortcuts)
			html += '</ul>'
			return html
		finally: self._release()

	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = _('<p><a href="%s">Click here to load a set of default shortcuts</a></p>') % self.getUrl('loaddefault_redir')
		html += '<table border=0 width="100%">'
		scshort = req.getFirstValue('short')
		sclong = ''
		for shortcut in self.shortcuts:
			if shortcut.short == scshort:
				sclong = shortcut.long
			html += '<tr><td colspan=2 class="whitebox">'
			html += '<table border=0 width="100%"><tr><td width="80%" class="smallfont">'
			html += '<b>"%s"</b> %s' % (shortcut.short, shortcut.long)
			html += '</td><td width="20%" align="right" valign="top" class="smallfont">'
			html += '%s<br>%s</td></tr>' % (
				'<a href="%s">%s</a>' % (req.getUrl('index_html', short=shortcut.short), _('edit')),
				'<a href="%s">%s</a>' % (req.getUrl('delshortcut_redir', short=shortcut.short), _('delete'))
			)
			html += '</table></td></tr>'
		html += '</table>'
		html += '<form action="%s" method="POST">' % req.getUrl('addshortcut_redir')
		html += '<table border=0 width="100%">'
		html += '<tr><td>%s</td><td>' %_('shortcut')
		html += '<input name="short" size=30 value="%s">' % scshort
		html += '</td></tr><tr><td>%s</td><td>' %_('shortcut<br>expansion')
		html += '<textarea name="long" cols=50 rows=5>%s</textarea>' % macros.quoteHTML(sclong)
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="ShortcutSubmit" value="%s">' % _('add')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		req.setLocalValue('title', _("Add or Delete Shortcuts"))
		return req.renderPage('BaseTemplate', body=html)
		
	def loaddefault_redir(self, req):
		for scshort in shorties.keys():
			self.addShortcut(scshort, shorties[scshort])
		return req.getUrl()
		
	def addshortcut_redir(self, req):
		scshort = req.getFirstValue('short')
		sclong = req.getFirstValue('long')
		self.addShortcut(scshort, sclong)
		return req.getUrl()
		
	def delshortcut_redir(self, req):
		scshort = req.getFirstValue('short')
		self.delShortcut(scshort)
		return req.getUrl()
	
	# add or change an existing shortcut
	def addShortcut(self, scshort, sclong):
		try:
			self._acquire()
			(res, found) = self.shortcuts.locate({'short':scshort})
			if found:
				row = self.shortcuts[res]
				row.long = sclong
			else:
				self.shortcuts.append({'short':scshort, 'long':sclong})
			self.shortcutcache[scshort] = sclong
			self._commit()
			return found
		finally: self._release()
		
	# delete an existing shortcut
	def delShortcut(self, scshort):
		try:
			self._acquire()
			(idx, found) = self.shortcuts.locate({'short':scshort})
			if found:
				self.shortcuts.delete(idx)
				self._commit()
				del self.shortcutcache[scshort]
			return found
		finally: self._release()
		
	# This method processes a text and replaces all shortcuts in that text.
	# It shouldn't be called on structured text, but on HTML postings.
	# You can give a different string as a quotedelimiter, if you need to.
	# If you pass in a request, that one will be used to locally resolve
	# the shortcut text. This is used when strict restructured Text is used.
	def processShortcuts(self, text, quotedelimiter='"', request=None):
		def replaceit(match):
			scshort = match.group(1)
			if self.shortcutcache.has_key(scshort):
				if request:
					return request.renderText(self.shortcutcache[scshort])
				else:
					return self.shortcutcache[scshort]
			else:
				return '"%s"' % scshort
		shortcutre = re.compile('%s(.+?)%s' % (quotedelimiter, quotedelimiter))
		return shortcutre.sub(replaceit, text)

	# This method returns the long text of a shortcut. Since reST
	# lowercases the shortcut text, both ways are tested. Exact
	# case match is faster, though. If the dynamic context
	# contains a request object, this is used to render the
	# shortcut source to get the result HTML output. Otherwise
	# the source is returned (and needs to be postprocessed)
	def queryShortcut(self, short):
		macros = self.getToolByNamespace('macros')
		if self.shortcutcache.has_key(short):
			if _flet.has('request'):
				return _flet.request.renderText(self.shortcutcache[short])
			else:
				return self.shortcutcache[short]
		shorty = short
		if type(shorty) == type(u''):
			shorty = short.encode(_PyDS.documentEncoding)
		if shorty.find('.') >= 0:
			(tool, pid) = shorty.split('.', 1)
			parms = []
			if pid.find('.') >= 0:
				parms = pid.split('.')
				pid = parms[0]
				del parms[0]
			tool = self.getToolByNamespace(tool)
			post = tool.getRSSItem(pid)
			if tool and post:
				return tool.getRSSItemLinkTag(post, pid, *parms).decode(_PyDS.documentEncoding)
		else:
			for tool in PyDS.Tool.getToolsWithMethod('getRSSItem'):
				post = tool.getRSSItem(shorty)
				if post:
					return tool.getRSSItemLinkTag(post, shorty).decode(_PyDS.documentEncoding)
		for key in self.shortcutcache.keys():
			if key.lower() == shorty:
				if _flet.has('request'):
					return _flet.request.renderText(self.shortcutcache[key])
				else:
					return self.shortcutcache[key]
		return ''

__desc__ = _("This tool allows to configure shortcuts for postings.")

PyDS.Tool.registerTool(ShortcutTool, 'shortcuts', _('Shortcuts'), __desc__, 550)

# Demonstration of how a tool can affect structured text rendering: 

class SubstitutionTransform(docutils.transforms.Transform):
	"""`Transform` to resolve stray substitutions."""

	_resolver = None

	# should run before all other transformations
	default_priority = 0
	
	def apply(self):
		"""Perform the transform."""
		resolver = SubstitutionTransform._resolver
		if resolver is None: 
			try: 
				shortcutTool = PyDS.Tool.getTool('shortcuts')
				resolver = SubstitutionTransform._resolver = shortcutTool.queryShortcut
			except KeyError: # ShortcutTool not yet registered with Tool? 
				return
		defs = self.document.substitution_defs
		for refname, refs in self.document.substitution_refs.items():
			if defs.has_key(refname): 
				continue
			subtext = resolver(refname)
			if not subtext: 
				continue
			for ref in refs: 
				sub = docutils.nodes.raw(rawsource = ref.rawsource, 
				                         text = subtext)
				sub['format'] = 'html'
				ref.parent.replace(ref, sub)
			del self.document.substitution_refs[refname]

PyDS.StructuredText.registerTransform(SubstitutionTransform)

