__doc__ = """

Strip-o-Gram HTML Conversion Library
(c) Chris Withers

This is open source. Make up whatever license you like but
I'll beat you with a big stick if I don't like it!

Use as follows:

from stripogram import html2text, html2safehtml

   mylumpofdodgyhtml # a lump of dodgy html ;-)

   mylumpofcoolcleancollectedhtml = html2safehtml(mylumpofdodgyhtml,valid_tags=('b', 'a', 'i', 'br', 'p'))

   mylumpoftext = html2text(mylumpofcoolcleancollectedhtml)

If you want more information on how it works, read the source :-)

Thanks to Itamar Shtull-Trauring for the original HTML Filter

Thanks to Andy McKay for the testing he has done.

"""

# $Id: Stripogram.py,v 1.5 2004/02/14 09:40:55 gb Exp $

import sgmllib, string

class HTML2Text(sgmllib.SGMLParser):
    
    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
    # patch apos back into the entitydefs (RSS and XHTML need it)
    entitydefs['apos'] = "'"
    
    def __init__(self):
        sgmllib.SGMLParser.__init__(self)
        self.result = ""
        self.indent = 0
        self.ol_number = 0

    def add_line(self,text,newline='\n'):
        self.result = self.result + self.indent*'   ' + text + newline

    def mod_indent(self,i):
        self.indent = self.indent + i
        if self.indent < 0:
            self.indent = 0
        
    def handle_data(self, data):
        if data:
            map(self.add_line,string.split(string.strip(data),'\n'))

    def unknown_starttag(self, tag, attrs):
        """ Convert HTML to something meaningful in plain text """
        tag = string.lower(tag)
        
        if tag[0]=='h' or tag in ['br','pre','p','hr']:
            # insert a blank line
            self.add_line('')
            
        elif tag =='img':
            # newline, text, newline
            src = ''
            
            for k, v in attrs:
                if string.lower(k) == 'src':
                    src = v
                    
            self.add_line('')
            self.add_line('Image: %s' % src)
            
        elif tag =='li':
            self.add_line('')
            if self.ol_number:
                # num - text
                self.add_line('%s - ' % self.ol_number,'')
                self.ol_number = self.ol_number + 1
            else:
                # - text
                self.add_line('- ','')
            
        elif tag in ['dd','dt']:
            self.add_line('')
            # increase indent
            self.mod_indent(+1)
            
        elif tag in ['ul','dl','ol','div']:
            # blank line
            #self.add_line('')
            # increase indent
            self.mod_indent(+1)
            if tag=='ol':
                self.ol_number = 1
                
    def unknown_endtag(self, tag):
        """ Convert HTML to something meaningful in plain text """
        tag = string.lower(tag)
        
        if tag[0]=='h' or tag in ['pre']:
            # newline, text, newline
            self.add_line('')
            
        elif tag =='li':
            #self.add_line('')
            pass
            
        elif tag in ['dd','dt']:
            #self.add_line('')
            # descrease indent
            self.mod_indent(-1)
            
        elif tag in ['ul','dl','ol','div']:
            # blank line
            #self.add_line('')
            # decrease indent
            self.mod_indent(-1)
            self.ol_number = 0
        
class HTML2SafeHTML(sgmllib.SGMLParser):
    
    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
    entitydefs['apos'] = "'"
    can_close   = ['li','p','dd','dt','option']
    never_close = ['br','wbr','hr','input','isindex','base','meta']
    
    def __init__(self,valid_tags):
        sgmllib.SGMLParser.__init__(self)
        self.valid_tags = valid_tags
        self.result = ""
        self.openTags = []
        
    def end_tag(self,tag):
        self.result = "%s</%s>" % (self.result, tag)
            
    def handle_data(self, data):
        if data:
            self.result = self.result + data

    def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)
        
    def handle_entityref(self, name):
        if self.entitydefs.has_key(name): 
            x = ';'
        else:
            # this breaks unstandard entities that end with ';'
            x = ''
        self.result = "%s&%s%s" % (self.result, name, x)

    def unknown_starttag(self, tag, attrs):
        """ Delete all tags except for legal ones """
        tag = string.lower(tag)
        if tag in self.valid_tags:
            
            self.result = self.result + '<' + tag
            
            for k, v in attrs:
                if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
                    self.result = '%s %s="%s"' % (self.result, k, v)
                    
            if tag not in self.never_close:
                self.openTags.append(tag)
                
            self.result = self.result + '>'
                
    def unknown_endtag(self, tag):
        tag = string.lower(tag)
        try:            

            while tag != self.openTags[-1] and self.openTags[-1] in self.can_close:
                self.openTags.pop()
            
            if tag==self.openTags[-1]:
                self.end_tag(self.openTags.pop())                
                
        except IndexError:
            pass

    def cleanup(self):
        """ Append missing closing tags """
        while self.openTags:
            tag = self.openTags.pop()
            if tag not in self.can_close:
                self.end_tag(tag)
        
def html2text(s):
    parser = HTML2Text()
    parser.feed(s)
    parser.close()
    return parser.result

def html2safehtml(s, valid_tags=('b', 'a', 'i', 'br', 'p')):
    parser = HTML2SafeHTML(valid_tags)
    parser.feed(s)
    parser.close()
    parser.cleanup()
    return parser.result

def test(function,input,expected):
    output = apply(function,input)
    if output==expected:
        print '.',
    else:
        print "\nInput   : '%s'\nExpected: '%s'\n     Got: '%s'\n" % (input,expected,output)
              
# tests
if __name__=='__main__':
    valid_tags=['b', 'a', 'i', 'br', 'p','ul','li']
    test(html2safehtml,('<b>x<br>y</b>z<p>a',),'<b>x<br>y</b>z<p>a')
    test(html2safehtml,('<b>x<br>y<i>z</b>a</i>b<p>',),'<b>x<br>y<i>za</i>b<p></b>')
    test(html2safehtml,('<b><p>x</p>y<p>z</p>a</b>',),'<b><p>x</p>y<p>z</p>a</b>')
    test(html2safehtml,('<p>x<i>y</p>z</i>a<p>b</p>',),'<p>x<i>yz</i>a<p>b</p>')
    test(html2safehtml,('<ul><li>x<li>y<li>z</ul>',valid_tags),'<ul><li>x<li>y<li>z</ul>')
           

