"""
Python Desktop Server - Translation support

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: Translation.py,v 1.5 2003/01/22 13:28:34 gb Exp $

import os
import sys
import gettext
import threading

from distutils.sysconfig import PREFIX

# find a translation file in several locations

def findTranslation(module, language):
	msgfn = '%s-%s.msgs' % (module, language)
	msgs = os.path.join(_PyDS.SHAREDDIR, msgfn)
	if os.path.exists(msgs):
		return msgs
	else:
		if os.path.exists(msgfn):
			return msgfn
	return None

# This class handles our own translation format

class MyTranslations(gettext.NullTranslations):

	def __init__(self, fp, module, lang):
		self._info = {}
		self._charset = None
		self._catalog = {}
		self._language = lang
		self._module = module
		self._lock = threading.RLock()
		if fp:
			self._parse(fp, lang)

	def _acquire(self):
		self._lock.acquire()

	def _release(self):
		self._lock.release()

	def _parse(self, fp, lang):
		self._catalog[lang] = catalog = {}
		line = fp.readline()
		while line:
			line = line.strip()
			if line and line[0] != '#':
				p = line.find('::=')
				if p > 0:
					msg1 = line[:p]
					msg2 = line[p+3:]
					if msg2:
						catalog[eval(msg1)] = eval(msg2)
			line = fp.readline()

	def gettext(self, msg, lang=None):
		try:
			self._acquire()
			if not(lang):
				lang = self._language
			if not(self._catalog.has_key(lang)):
				msgfn = findTranslation(self._module, lang)
				try:
					fp = open(msgfn)
					self._parse(fp, lang)
					fp.close()
				except:
					self._catalog[lang] = {}
			return self._catalog[lang].get(msg,msg)
		finally: self._release()

	def ugettext(self, message, lang=None):
		tmsg = self.gettext(message, lang)
		return unicode(tmsg, self._charset)
		
	def switch(self, lang):
		try:
			self._acquire()
			self._language = lang
		finally: self._release()

# return a translation object for a given language. If possible use
# MyTranslations, but if not possible fall back to NullTranslations.

def translation(module, language):
	msgs = findTranslation(module, language)
	if msgs:
		try:
			msgf = open(msgs)
			trns = MyTranslations(msgf,module,language)
			msgf.close()
			return trns
		except:
			print "Problem with translation for %s" % language
			return gettext.NullTranslations()
	else:
		return gettext.NullTranslations()


if __name__ == '__main__':
	translation('pyds', 'en').install()
	print _('Hello World!')
else:
	tr = translation('pyds', _PyDS.language)
	_PyDS.translation = tr
	tr.install()


