"""

Python Desktop Server - XML-RPC handler module

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: XMLRPCHandler.py,v 1.18 2004/09/12 16:12:39 gb Exp $

import sys
import traceback
import xmlrpclib

import PyDS.Tool

from medusa import xmlrpc_handler

def _dir(*params):
	"Backing command for XML-RPC dir() command"
	if len(params)<1: 
		return map(lambda t: t.name, PyDS.Tool.getTools())
	elif len(params)==1: 
		namespace = params[0]
		if PyDS.Tool.getTool(namespace): 
			return PyDS.Tool.getTool(namespace).listRegisteredFunctions()
		else: 
			raise _("Invalid namespace"), namespace
	else: 
		raise _("dir() takes only one [optional] argument")

class xmlrpc_handler(xmlrpc_handler.xmlrpc_handler):

	def match(self, request):
		prefs = PyDS.Tool.getTool('preferences')
		if prefs.get('RPC', 'enabled'):
			if request.uri[:5] == '/RPC2':
				return 1
			else:
				return 0
		else:
			return 0
			
	def continue_request(self, data, request):

		params, method = xmlrpclib.loads(data)

		path, qparams, query, qfragmet = request.split_uri()
		if path[:6] == '/RPC2/':
			method = '%s.%s' % (path[6:], method)

		try:
			response = self.call(method, params)
			if type(response) != type(()):
				response = (response,)
		except:
			response = xmlrpclib.dumps(
				xmlrpclib.Fault(1, "%s:%s" % (sys.exc_type, sys.exc_value)),
				encoding=_PyDS.documentEncoding
			)
			(e, d, tb) = sys.exc_info()
			msg = _('XML-RPC call exception %s: %s') % (e, d)
			for row in traceback.extract_tb(tb):
				(file, line, function, source) = row
				msg += '<br>%s[%s] in %s' % (file, line, function)
			if type(msg) == type(u''):
				msg = msg.encode('_PyDS.documentEncoding')
			PyDS.Tool.log(msg)
		else:
			response = xmlrpclib.dumps(
				response,
				methodresponse=1,
				encoding=_PyDS.documentEncoding
			)
				
		request['Content-Type'] = 'text/xml'
		request['Content-length'] = len(response)
		request.push(response)
		request.done()

	def call(self, method, params):
		prefs = PyDS.Tool.getTool('preferences')
		if type(method) == type(u''):
			method = method.encode(_PyDS.documentEncoding)
		newparams = []
		for p in params:
			if type(p) == type(u''):
				newparams.append(p.encode(_PyDS.documentEncoding))
			elif isinstance(p, xmlrpclib.Binary):
				newparams.append(p.data)
			else:
				newparams.append(p)
		params = newparams

		try: 
			(base, method) = method.split('.')
		except ValueError: 
			if method == 'dir':
				return _dir(*params)

		authuser = ''

		if method not in ['listRegisteredFunctions'] and not(base in ['blogger', 'metaWeblog']):
			if len(params) < 2:
				raise _("Too few arguments - need at least user and password"), len(params)
			if prefs.get('RPC', 'username') != params[0]:
				raise _("Authentication failure"), params[0]
			if prefs.get('RPC', 'password') != params[1]:
				raise _("Authentication failure"), params[0]
			authuser = params[0]
			params = params[2:]

		if PyDS.Tool.getTool(base):
			tool = PyDS.Tool.getTool(base)
			try:
				_flet.begin(rpc=1, current=tool, authenticated_user=authuser)
				if hasattr(tool, method):
					if _PyDS.verbose:
						print " ... XMLRPC %s.%s" % (
							tool.name,
							method
						)
					meth = getattr(tool, method)
					return apply(meth, params)
				else:
					raise _("Method not found in namespace %s") % base, method
			finally:
				_flet.end()
		else:
			raise _("Namespace not found"), base

