"""

Python Desktop Server - Magazin Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: ZineTool.py,v 1.12 2004/02/04 09:56:39 gb Exp $

import time
import sys

import PyDS.Tool

class ZineTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("issues[id:I,pubtime:F,title:S,desc:S,text:S,published:I]")
		db.getas("articles[issue:I,id:I,pubtime:F,title:S,desc:S,text:S]")
		db.getas("prefs[maxid:I,title:S,desc:S]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.issues = self.db.view('issues').ordered(1)
			self.articles = self.db.view('articles').ordered(2)
			self.prefs = self.db.view('prefs')
			if len(self.prefs) == 0:
				self.prefs.append({'maxid':0})
			self._commit()
		finally: self._release()
	
	def _backup(self):
		l = self._backupTable('prefs')
		l.extend(self._backupTable('issues', sortcols=['id']))
		l.extend(self._backupTable('articles', sortcols=['issue','id']))
		return l

	def _initopts(self):
		self.rss = 1
	
	def _initactive(self):
		navigation = self.getToolByNamespace('navigation')
		navigation._addHook('select', 'issue',
			_('Link to a zine issue'),
			self._issueNavHook,
			self.hasIssues)
		navigation._addHook('select', 'issueindex',
			_('Link to the zine issue index'),
			self._issueIndexNavHook,
			self.hasIssues)
		themes = self.getToolByNamespace('themes')
		themes.hookTemplate(
			'ZineTemplate',
			_('zine template for the desktop website'),
			'templates/ZineTemplate.tmpl'
		)
		themes.hookTemplate(
			'ZineRendering',
			_('zine template for the cloud'),
			'templates/ZineRendering.tmpl'
		)
		themes.hookTemplate(
			'ZineArticleTemplate',
			_('zine article template for the desktop website'),
			'templates/ZineArticleTemplate.tmpl'
		)
		themes.hookTemplate(
			'ZineArticleRendering',
			_('zine article template for the cloud'),
			'templates/ZineArticleRendering.tmpl'
		)

	def _getNewId(self):
		try:
			self._acquire()
			self.prefs[0].maxid += 1
			self._commit()
			return self.prefs[0].maxid
		finally: self._release()

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d issues in the database</li>') % len(self.issues)
			html += _('<li>There are %d articles in the database</li>') % len(self.articles)
			html += _('<li>There are %d issues in the render queue</li>') % self.queue.length()
			html += '</ul>'
			return html
		finally: self._release()

	def _issueNavHook(self, qual, value, preset=None):
		if qual == 'html':
			html = '<select name="%s">' % value
			for issue in self.getIssues():
				selected = ''
				if preset and int(issue['id']) == int(preset):
					selected = ' selected'
				html += '<option value="%s"%s>%s' % (
					issue['id'],
					selected,
					issue['title']
				)
			html += '</select>'
			return html
		elif qual == 'url':
			return self.getIssueUrl(value)
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	def _issueIndexNavHook(self, qual, value, preset=None):
		if qual == 'html':
			return ''
		elif qual == 'url':
			return self.getStaticUrl()
		else:
			raise ValueError('%s is not a valid qualifier' % qual)
	
	# this method returns the title of a page for rendering
	def getPageTitle(self, context):
		try:
			self._acquire()
			title = PyDS.Tool.StandardTool.getPageTitle(self, context)
			if self.prefs[0].title:
				title = self.prefs[0].title
			if context.get('issueid',''):
				if context.get('articleid',''):
					article = self.getArticle(int(context['issueid']), int(context['articleid']))
					if article:
						title = article['title']
				else:
					issue = self.getIssue(int(context['issueid']))
					if issue:
						title = issue['title']
			return title
		finally: self._release()

	# this method returns the title and description of a page for rendering
	def getPageDescription(self, context):
		try:
			self._acquire()
			desc = PyDS.Tool.StandardTool.getPageDescription(self, context)
			if self.prefs[0].desc:
				desc = self.prefs[0].desc
			if context.get('issueid',''):
				if context.get('articleid',''):
					article = self.getArticle(int(context['issueid']), int(context['articleid']))
					if article:
						desc = article['desc']
				else:
					issue = self.getIssue(int(context['issueid']))
					if issue:
						desc = issue['desc']
			return desc
		finally: self._release()
	
	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		t = _('Click to add a new zine issue')
		html = '<p><a href="%s" title="%s">%s</a>' % (
			self.getUrl('edit_html'), t, t
		)
		html += "<ul>"
		for issue in self.getIssues():
			draftimg = ''
			if not(issue['published']):
				draftimg = '<img src="/images/new.gif" alt="%s">' % _('this issue is in draft mode')
			html += '<li>%s<b><a href="%s">%s (%d)</a>:</b> %s <span class="smallfont">(%s %s)</span></li>' % (
				draftimg,
				self.getIssueUrl(issue['id']),
				issue['title'],
				issue['id'],
				issue['desc'],
				_('last change'),
				macros.Today(issue['pubtime'], withtime=1),
			)
		html += "</ul>"
		req.setLocalValue('title', self.getPageTitle({}))
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def show_html(self, req):
		macros = self.getToolByNamespace('macros')
		sid = int(req.getFirstValue('id'))
		issue = self.getIssue(sid)
		issue['rendered'] = req.renderTextWithTraceback(
			issue['text'],
			structured=1,
			issue=issue,
			homeTool=self,
			uniqueId=str(sid)
		)
		issue['staticurl'] = self.getStaticUrl(page='index.html', category=str(sid))
		html = req.renderPage('ZineTemplate',
			issue=issue
		)
		req.setLocalValue('title', issue['title'])
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		if id:
			id = int(id)
		else:
			id = -1
		issue = self.getIssue(id)
		title = _('place your zine issue title here')
		text = _('Place your zine issue text here.\n\nThis is above the content.\n\n$zine.toc($issue)\n\nThis is below the content.')
		desc = _('place a short description of your zine issue here')
		published = 0
		if issue:
			title = issue['title']
			text = issue['text']
			desc = issue['desc']
		html = '<form action="%s" method="POST">' % self.getUrl('edit_redir')
		if issue:
			html += '<input type="hidden" name="id" value="%d">' % id
		html += '<table border=0>'
		html += '<tr>'
		if issue:
			html += '<th align="right">%s</th><td> %d</td>' % (_('Issue'), id)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><input name="title" value="%s" size=50></td>' % (
			_('Title'), title
		)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td>' % _('Published')
		html += macros.checkBox('published', published)
		html += '</td>'
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="desc" rows=5 cols=60>%s</textarea></td>' % (
			_('Description'), macros.quoteHTML(desc)
		)
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="text" rows=30 cols=60>%s</textarea></td>' % (
			_('Text'), macros.quoteHTML(text)
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="IssueDelete" value="%s">' % _('delete')
		html += ' <input type=submit name="IssueSave" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different issue')
		)
		req.setLocalValue('title', title)
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def edit_redir(self, req):
		id = req.getFirstValue('id')
		if req.getFirstValue('IssueDelete'):
			self.deleteIssue(int(id))
			return self.getUrl()
		else:
			if id:
				id = int(id)
			else:
				id = -1
			title = req.getFirstValue('title')
			text = req.getFirstValue('text')
			desc = req.getFirstValue('desc')
			published = int(req.getFirstValue('published'))
			if title and text:
				id = self.addIssue(id, title, text, desc, published)
				return self.getUrl('show_html', id=str(id))
			else:
				if id:
					return self.getUrl('edit_html', id=str(id), errmsg=_('You need to give title and text!'))
				else:
					return self.getUrl('edit_html', errmsg=_('You need to give title and text!'))

	def articleshow_html(self, req):
		macros = self.getToolByNamespace('macros')
		issueid = int(req.getFirstValue('issueid'))
		sid = int(req.getFirstValue('id'))
		article = self.getArticle(issueid, sid)
		issue = self.getIssue(issueid)
		article['rendered'] = req.renderTextWithTraceback(
			article['text'],
			structured=1,
			issue=issue,
			article=article,
			homeTool=self,
			uniqueId=str(issueid)+':'+str(sid)
		)
		article['staticurl'] = self.getStaticUrl(page=('%d.html' % sid), category=str(issueid))
		html = req.renderPage('ZineArticleTemplate',
			issue=issue,
			article=article
		)
		req.setLocalValue('title', article['title'])
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def articleedit_html(self, req):
		macros = self.getToolByNamespace('macros')
		issueid = int(req.getFirstValue('issueid'))
		id = req.getFirstValue('id')
		if id: id = int(id)
		else: id = -1
		article = self.getArticle(issueid,id)
		title = _('place your zine article title here')
		text = _('place your zine article text here')
		desc = _('place a short description of your zine article here')
		if article:
			title = article['title']
			text = article['text']
			desc = article['desc']
		html = '<form action="%s" method="POST">' % self.getUrl('articleedit_redir')
		if article:
			html += '<input type="hidden" name="id" value="%d">' % id
		html += '<input type="hidden" name="issueid" value="%d">' % issueid
		html += '<table border=0>'
		html += '<tr>'
		if article:
			html += '<th align="right">%s</th><td> %d/%d</td>' % (_('Article'), issueid, id)
		html += '</tr><tr>'
		html += '<th align="right">%s</th><td><input name="title" value="%s" size=50></td>' % (
			_('Title'), title
		)
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="desc" rows=5 cols=60>%s</textarea></td>' % (
			_('Description'), macros.quoteHTML(desc)
		)
		html += '</tr><tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="text" rows=30 cols=60>%s</textarea></td>' % (
			_('Text'), macros.quoteHTML(text)
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="ArticleDelete" value="%s">' % _('delete')
		html += ' <input type=submit name="ArticleSave" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl('show_html', id=issueid),
			_('choose a different article')
		)
		req.setLocalValue('title', title)
		return req.renderPage('BaseTemplate',
			body=html
		)
		
	def articleedit_redir(self, req):
		issueid = int(req.getFirstValue('issueid'))
		id = req.getFirstValue('id')
		if req.getFirstValue('ArticleDelete'):
			self.deleteArticle(issueid,int(id))
			return self.getUrl()
		else:
			if id: id = int(id)
			else: id = -1
			title = req.getFirstValue('title')
			text = req.getFirstValue('text')
			desc = req.getFirstValue('desc')
			if title and text:
				id = self.addArticle(issueid, id, title, text, desc)
				return self.getUrl('articleshow_html', issueid=str(issueid), id=str(id))
			else:
				if id:
					return self.getUrl('articleedit_html', issueid=str(issueid), id=str(id), errmsg=_('You need to give title and text!'))
				else:
					return self.getUrl('articleedit_html', issueid=str(issueid), errmsg=_('You need to give title and text!'))

	def getPreferencePanels(self):
		return [
			['issues_panel', _('Issues'),
				_('Here you can configure the title and description of the zine issues index page.')]
		]
			
	def issues_panel(self, req):
		macros = self.getToolByNamespace('macros')
		try:
			self._acquire()
			html = '<p>%s' % _('Here you can configure the title and description of the zine issues index page.')
			html += '<form action="%s" method="POST">' % self.getUrl('issues_redir')
			html += '<table border=0 width="100%">'
			html += '<tr><td>%s</td><td>' %_('title')
			html += '<input name="title" size=50 value="%s">' % self.prefs[0].title
			html += '</td></tr><tr><td>%s</td><td>' % _('description')
			html += '<textarea name="desc" rows=5 cols=50>%s</textarea>' % macros.quoteHTML(self.prefs[0].desc)
			html += '</td></tr><tr><td colspan=2 align="right">'
			html += '<input type=submit name="IssuesSubmit" value="%s">' % _('save')
			html += '</td></tr>'
			html += '</table>'
			html += '</form>'
			return html
		finally: self._release()
		
	def issues_redir(self, req):
		title = req.getFirstValue('title')
		desc = req.getFirstValue('desc')
		try:
			self._acquire()
			self.prefs[0].title = title
			self.prefs[0].desc = desc
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='issues_panel',
				fortool='zine'
			)
		finally: self._release()
		
	def _renderIssueOnline(self, id, dummy):
		macros = self.getToolByNamespace('macros')
		issue = self.getIssue(id)
		if issue:
			req = self._getDummyRequest(now=issue['pubtime'])
			req.setLocalValue('title', issue['title'])
			issue['rendered'] = req.renderText(
				issue['text'],
				structured=1,
				issue=issue,
				homeTool=self,
				uniqueId=str(id)
			)
			html = req.renderPage(
				'ZineRendering',
				issue=issue
			)
			src = req.renderPage(
				'BaseRendering',
				issueid = id,
				body = html,
				category = ''
			)
			self.storeHTML(src, page='index.html', category=str(id))
			for article in self.getFullArticles(id):
				req.setLocalValue('title', article['title'])
				article['rendered'] = req.renderText(
					article['text'],
					structured=1,
					issue=issue,
					article=article,
					homeTool=self,
					uniqueId=str(id)+':'+str(article['id'])
				)
				html = req.renderPage(
					'ZineArticleRendering',
					issue=issue,
					article=article,
				)
				src = req.renderPage(
					'BaseRendering',
					issueid=id,
					articleid=article['id'],
					body = html,
					category = ''
				)
				self.storeHTML(src, page='%d.html' % article['id'], category=str(id))

	# this method puts a render request into the queue
	def renderIssue(self, id):
		issue = self.getIssue(id)
		if issue['published']:
			self.queue.append(self._renderIssueOnline, id, None)
		else:
			self.queue.append(self._deleteRenderedIssue, id, None)
		return issue['published']

	def _rerender(self):
		try:
			self._acquire()
			for issue in self.issues:
				self.renderIssue(issue.id)
		finally: self._release()

	def _renderIndexHook(self):
		macros = self.getToolByNamespace('macros')
		req = self._getDummyRequest()
		html = "<ul>"
		for issue in self.getIssues():
			if issue['published']:
				html += '<li><a href="%s">%s</a> <span class="smallfont">(%s %s)</span></li>' % (
					self.getStaticUrl(page='index.html', category=issue['id']),
					issue['title'],
					_('last change'),
					macros.Today(issue['pubtime'], withtime=1)
				)
		html += "</ul>"
		src = req.renderPage(
			'BaseRendering',
			body = html,
			category = ''
		)
		self.storeHTML(src)
	
	def _deleteRenderedIssue(self, id, dummy):
		self.deleteFile('%d/index.html' % id)
		for article in self.getArticles(id):
			self.deleteFile('%d/%d.html' % (id, article['id']))

	def _deleteRenderedArticle(self, key, dummy):
		(issueid, id) = key
		self.deleteFile('%d/%d.html' % (issueid, id))

	# this method deletes an existing zine issue and drops the rendered
	# output
	def deleteIssue(self, id):
		try:
			self._acquire()
			(idx, found) = self.issues.locate({'id':id})
			if found:
				self.queue.append(
					self._deleteRenderedIssue,
					id,
					None
				)
				self.issues.delete(idx)
				liste = []
				idx = 0
				for article in self.articles:
					if article.issue == id:
						liste.append(idx)
						self.queue.append(
							self._deleteRenderedArticle,
							(id, article.id),
							None
						)
					idx += 1
				liste.reverse()
				for idx in liste:
					self.articles.delete(idx)
				self._commit()
			return found
		finally: self._release()

	# this methods adds a new issue or changes an old issue
	def addIssue(self, id, title, text, desc, published):
		macros = self.getToolByNamespace('macros')
		try:
			self._acquire()
			req = self._getDummyRequest()
			pubtime = time.time()
			(idx, found) = self.issues.locate({'id':id})
			if found:
				issue = self.issues[idx]
				if issue.title != title or issue.text != text or issue.desc != desc or issue.published != published:
					issue.title = title
					issue.text = text
					issue.pubtime = pubtime
					issue.desc = desc
					issue.published = published
					self._commit()
					self.renderIssue(id)
			else:
				id = self._getNewId()
				self.issues.append({
					'id': id,
					'title': title,
					'text': text,
					'pubtime': pubtime,
					'desc': desc,
					'published': published,
				})
				self._commit()
				self.renderIssue(id)
			return id
		finally:
			self._release()
	
	# do we have some issues defined?
	def hasIssues(self):
		try:
			self._acquire()
			return len(self.issues)
		finally:
			self._release()
	
	def _issueHash(self, issue, short=0):
		hash = {
			'id': issue.id,
			'title': issue.title,
			'desc': issue.desc,
			'published': issue.published,
			'pubtime': issue.pubtime
		}
		if short == 0:
			hash['text'] = issue.text
		return hash
	
	# this method returns a issue as a dictionary
	def getIssue(self, id):
		try:
			self._acquire()
			id = int(id)
			(idx, found) = self.issues.locate({'id':id})
			if found:
				issue = self.issues[idx]
				return self._issueHash(issue)
			else:
				return {}
		finally: self._release()
	
	# return a list of all defined issues (all data sans the issue text)
	def getIssues(self):
		try:
			self._acquire()
			liste = []
			for issue in self.issues:
				liste.append(self._issueHash(issue, short=1))
			return liste
		finally: self._release()
	
	# this method deletes an existing zine article and drops the rendered
	# output
	def deleteArticle(self, issueid, id):
		try:
			self._acquire()
			(idx, found) = self.articles.locate({'issue':issueid, 'id':id})
			if found:
				self.articles.delete(idx)
				self._commit()
				self.queue.append(
					self._deleteRenderedArticle,
					(issueid, id),
					None
				)
			return found
		finally: self._release()

	# this methods adds a new article or changes an old article
	def addArticle(self, issueid, id, title, text, desc):
		macros = self.getToolByNamespace('macros')
		try:
			self._acquire()
			req = self._getDummyRequest()
			pubtime = time.time()
			(idx, found) = self.articles.locate({'issue':issueid, 'id':id})
			if found:
				article = self.articles[idx]
				if article.title != title or article.text != text or article.desc != desc:
					article.title = title
					article.text = text
					article.pubtime = pubtime
					article.desc = desc
					self._commit()
					self.renderIssue(issueid)
			else:
				id = self._getNewId()
				self.articles.append({
					'issue': issueid,
					'id': id,
					'title': title,
					'text': text,
					'pubtime': pubtime,
					'desc': desc,
				})
				self._commit()
				self.renderIssue(issueid)
			return id
		finally:
			self._release()
	
	def _articleHash(self, article, short=0):
		hash = {
			'id': article.id,
			'title': article.title,
			'desc': article.desc,
			'pubtime': article.pubtime
		}
		if short == 0:
			hash['text'] = article.text
		return hash
	
	# this method returns a article as a dictionary
	def getArticle(self, issueid, id):
		try:
			self._acquire()
			id = int(id)
			(idx, found) = self.articles.locate({'issue':issueid,'id':id})
			if found:
				article = self.articles[idx]
				return self._articleHash(article)
			else:
				return {}
		finally: self._release()
	
	# return a list of all defined articles (all data sans the article text)
	def getArticles(self, issueid):
		try:
			self._acquire()
			liste = []
			for article in self.articles.select({'issue':issueid}):
				liste.append(self._articleHash(article, short=1))
			return liste
		finally: self._release()
	
	# return a list of all defined articles (full data)
	def getFullArticles(self, issueid):
		try:
			self._acquire()
			liste = []
			for article in self.articles.select({'issue':issueid}):
				liste.append(self._articleHash(article))
			return liste
		finally: self._release()
	
	# this method returns the URL for a article depending on context
	def getIssueUrl(self, id):
		if _flet.desktop:
			return self.getUrl('show_html', id=str(id))
		else:
			return self.getStaticUrl(page='index.html', category=str(id))
		
	# this method returns the URL for a issue depending on context
	def getArticleUrl(self, issueid, id):
		if _flet.desktop:
			return self.getUrl('articleshow_html', issueid=str(issueid), id=str(id))
		else:
			return self.getStaticUrl(page='%s.html' % id, category=str(issueid))

	# this method builds an HTML table of contents for articles in a zine
	def toc(self, issue):
		html = '<ul>'
		for article in self.getArticles(issue['id']):
			html += '<li><a href="%s">%s</a>: %s</li>' % (
				self.getArticleUrl(issue['id'], article['id']),
				article['title'],
				article['desc']
			)
		html += '</ul>'
		return html
		
	# tools wanting to catch an unresolved StructuredText link should return a 
	# CSS class and cloud URL from this method
	def getCloudUrlMatchingStrayLink(self, shortcut): 
		shortcut = shortcut.lower()
		try:
			self._acquire()
			for issue in self.issues: 
				if (issue.title.lower().find(shortcut) >= 0 or str(issue.id) == shortcut) and issue.published:
					break 
			else: # no match
				return ()
			return ('issue', self.getIssueUrl(issue.id))
		finally:
			self._release()
		
__desc__ = _("This is a tool that manages online zines (articles bundled to issues).")

PyDS.Tool.registerTool(ZineTool, 'zine', _('Zine Issues'), __desc__, 830)

