"""

Python Desktop Server - New Tool Template

!! This Tool doesn't have any function at all, it is just there to show you what
!! functions are needed to overwrite and overload and how to create a new tool.
!! This is just some small part of what can be done. For even more samples
!! how to build tools, have a look at the standard tools in the distribution,
!! most noteably the WeblogTool.py and the NavigationTool.py!

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: NewTool.py,v 1.11 2004/01/28 14:32:14 gb Exp $

# First you have to import the StandardTool class

import PyDS.Tool

# Some documentation about tool levels:
# 0 is the default tool. It is linked in the main toolbar to the left
# 100-799 are tools on the main toolbar, ordered by level
# 800-899 are tools on the second toolbar below the calendar
# 900-999 are tools on the third toolbar below the second toolbar
# 1000 and up are preferences tools

# To create a new LinkTool just instantiate it, no need to create Subclass. This is
# uncommented, so it isn't installed by the server.

#PyDS.Tool.registerTool(LinkTool, 'muensterland',
#	title=_('Weblogserver Muensterland'),
#	desc=_('This links to my Python Community Server homepage.'),
#	url='http://muensterland.org/',
#	level=890)

# To make your own simple tool like a LinkTool inherit from it
class BoldLinkTool(PyDS.Tool.LinkTool):

	# overload the nugget_html, as that is the only method usually invoked
	def nugget(self, url=None):
		if not(url):
			url = self.url
		html = '<a href="%s"><b>%s</b></a>: %s' % (url, self.title, self.desc)
		return html
		
# To make a standard tool, inherit from StandardTool
class MyCoolTool(PyDS.Tool.StandardTool):

	# if it uses database tables, overload this method. If you change the database,
	# remember to call self._commit() to commit changes to the database.
	# all functions that change object state or access databases must
	# be protected by the object lock with self._acquire()
	# and self._release()
	def _initdb(self):
		try:
			self._acquire()
			self.stuff = self.db.getas('stuff[id:S,col1:S,col2:S,col3:I]').ordered(1)
		finally: self._release()
		
	# this method can override options of a tool, like the marker wether it has rss rendering
	def _initopts(self):
		self.rss = 0
	
	# this method is called when no special method is given
	def index_html(self, req):
		html += "<h1>All Cool Tools</h1><ul>"
		for tool in getTools():
			html += '<li>%s</li>' % tool.nugget()
		html += "</ul>"
		return req.renderPage('BaseTemplate', body=html)
		
	# this is a redirector page - it usually is called in forms to store stuff
	# it does it's thing and then just returns the URL to jump to. Usually this
	# is your own tool url
	def edit_redir(self, req):
		return req.getUrl()

	# overload these three if you have date-related data to make the
	# calendar intelligent. They should check against your database tables
	# to see whethere data is available or not. Beyond and Upto are inclusive,
	# so you usually get passed the following month!
	def hasDataUpto(self, year, month, day=31):
		return 0

	def hasDataBeyond(self, year, month, day=1):
		return 0

	def hasDataForDay(self, year, month, day):
		return 0
		
# create a template, if it isn't there. This is to enable you to include templates
# in-source  instead of as separate files. This is commented out to not accidentally
# install bogus templates
#PyDS.Tool.spillOutTemplate("NewToolTemplate","""
#<html><head><title>$title</title></head>
#<body>
#$body
#</body>
#</html>
#""")

# Now set up a description. This can contain HTML, if you need to.
__desc__ = _("This is my really cool tool.")

# Register the tool to the registry. This is automatically on import. The
# import happens either directly in PyDS.Server or via automatic loading
# from the Tools directory. This call is commented out in this tool, so it doesn't
# show up in the toolbars. You should enable it to enable this tool.

#PyDS.Tool.registerTool(MyCoolTool, 'cool', _('CoolTool'), __desc__, 950)

